#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# hack to make sure -m transformer/generator works as expected
from typing import Optional
from parlai.core.params import ParlaiParser
from parlai.core.opt import Opt
from .modules import TransformerEncoder
from .modules import get_n_positions_from_options
from parlai.core.torch_ranker_agent import TorchRankerAgent
from .transformer import TransformerRankerAgent
from parlai.utils.torch import concat_without_padding
import torch


class CrossencoderAgent(TorchRankerAgent):
    """
    Equivalent of bert_ranker/crossencoder but does not rely on an external library
    (hugging face).
    """

    @classmethod
    def add_cmdline_args(
        cls, parser: ParlaiParser, partial_opt: Optional[Opt] = None
    ) -> ParlaiParser:
        """
        Add command-line arguments specifically for this agent.
        """
        TransformerRankerAgent.add_cmdline_args(parser, partial_opt=partial_opt)
        parser.set_defaults(encode_candidate_vecs=False)
        return parser

    def build_model(self, states=None):
        return CrossEncoderModule(self.opt, self.dict, self.NULL_IDX)

    def vectorize(self, *args, **kwargs):
        """
        Add the start and end token to the text.
        """
        kwargs['add_start'] = True
        kwargs['add_end'] = True
        obs = super().vectorize(*args, **kwargs)
        return obs

    def _set_text_vec(self, *args, **kwargs):
        """
        Add the start and end token to the text.
        """
        obs = super()._set_text_vec(*args, **kwargs)
        if 'text_vec' in obs and 'added_start_end_tokens' not in obs:
            obs.force_set(
                'text_vec', self._add_start_end_tokens(obs['text_vec'], True, True)
            )
            obs['added_start_end_tokens'] = True
        return obs

    def score_candidates(self, batch, cand_vecs, cand_encs=None):
        if cand_encs is not None:
            raise Exception(
                'Candidate pre-computation is impossible on the ' 'crossencoder'
            )
        num_cands_per_sample = cand_vecs.size(1)
        bsz = cand_vecs.size(0)
        text_idx = (
            batch.text_vec.unsqueeze(1)
            .expand(-1, num_cands_per_sample, -1)
            .contiguous()
            .view(num_cands_per_sample * bsz, -1)
        )
        cand_idx = cand_vecs.view(num_cands_per_sample * bsz, -1)
        tokens, segments = concat_without_padding(
            text_idx, cand_idx, self.use_cuda, self.NULL_IDX
        )
        scores = self.model(tokens, segments)
        scores = scores.view(bsz, num_cands_per_sample)
        return scores


class CrossEncoderModule(torch.nn.Module):
    """
    A simple wrapper around the transformer encoder which adds a linear layer.
    """

    def __init__(self, opt, dict, null_idx):
        super(CrossEncoderModule, self).__init__()
        n_positions = get_n_positions_from_options(opt)
        embeddings = torch.nn.Embedding(
            len(dict), opt['embedding_size'], padding_idx=null_idx
        )
        torch.nn.init.normal_(embeddings.weight, 0, opt['embedding_size'] ** -0.5)
        self.encoder = TransformerEncoder(
            n_heads=opt['n_heads'],
            n_layers=opt['n_layers'],
            embedding_size=opt['embedding_size'],
            ffn_size=opt['ffn_size'],
            vocabulary_size=len(dict),
            embedding=embeddings,
            dropout=opt['dropout'],
            attention_dropout=opt['attention_dropout'],
            relu_dropout=opt['relu_dropout'],
            padding_idx=null_idx,
            learn_positional_embeddings=opt['learn_positional_embeddings'],
            embeddings_scale=opt['embeddings_scale'],
            reduction_type=opt.get('reduction_type', 'first'),
            n_positions=n_positions,
            n_segments=2,
            activation=opt['activation'],
            variant=opt['variant'],
            output_scaling=opt['output_scaling'],
        )
        self.linear_layer = torch.nn.Linear(opt['embedding_size'], 1)

    def forward(self, tokens, segments):
        """
        Scores each concatenation text + candidate.
        """
        encoded = self.encoder(tokens, None, segments)
        res = self.linear_layer(encoded)
        return res



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.
#
# Download and build the data if it does not exist.

import parlai.core.build_data as build_data
import os
import json
import re
from parlai.utils.io import PathManager
from parlai.core.build_data import DownloadableFile

RESOURCES = [
    DownloadableFile(
        'https://raw.githubusercontent.com/salesforce/decanlp/d594b2bf127e13d0e61151b6a2af3bf63612f380/local_data/schema.txt',
        'schema.txt',
        '31da9bee05796bbe0f6c957f54d1eb82eb5c644a8ee59f2ff1fa890eff3885dd',
        zipped=False,
    )
]


def build(opt):
    dpath = os.path.join(opt['datapath'], 'MWSC')
    version = 'None'

    if not build_data.built(dpath, version_string=version):
        print('[building data: ' + dpath + ']')
        if build_data.built(dpath):
            # An older version exists, so remove these outdated files.
            build_data.remove_dir(dpath)
        build_data.make_dir(dpath)

        # Download the data.
        for downloadable_file in RESOURCES:
            downloadable_file.download_file(dpath)

        pattern = '\\[.*\\]'

        def get_both_schema(context):
            variations = [x[1:-1].split('/') for x in re.findall(pattern, context)]
            splits = re.split(pattern, context)
            results = []
            for which_schema in range(2):
                vs = [v[which_schema] for v in variations]
                context = ''
                for idx in range(len(splits)):
                    context += splits[idx]
                    if idx < len(vs):
                        context += vs[idx]
                results.append(context)
            return results

        schemas = []
        with PathManager.open(
            os.path.join(dpath, RESOURCES[0].file_name)
        ) as schema_file:
            schema = []
            for line in schema_file:
                if len(line.split()) == 0:
                    schemas.append(schema)
                    schema = []
                    continue
                else:
                    schema.append(line.strip())

        examples = []
        for schema in schemas:
            context, question, answer = schema
            contexts = get_both_schema(context)
            questions = get_both_schema(question)
            answers = answer.split('/')
            for idx in range(2):
                answer = answers[idx]
                question = questions[idx] + ' {} or {}?'.format(answers[0], answers[1])
                examples.append(
                    {'context': contexts[idx], 'question': question, 'answer': answer}
                )

        traindev = examples[:-100]
        test = examples[-100:]
        train = traindev[:80]
        dev = traindev[80:]

        splits = ['train', 'validation', 'test']
        for split, examples in zip(splits, [train, dev, test]):
            split_fname = '{}.json'.format(split)
            with PathManager.open(os.path.join(dpath, split_fname), 'a') as split_file:
                for ex in examples:
                    split_file.write(json.dumps(ex) + '\n')

        # Mark the data as built.
        build_data.mark_done(dpath, version_string=version)



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.
from typing import Optional
from parlai.core.params import ParlaiParser
from parlai.core.opt import Opt
from abc import ABC, abstractmethod
import os
import random

from parlai.core.message import Message
from parlai.core.teachers import FixedDialogTeacher

from .build import build

SINGLE_TURN_DATA = 'single_turn_safety.json'


OK_CLASS = '__ok__'
NOT_OK_CLASS = '__notok__'


class _BaseSafetyTeacher(FixedDialogTeacher, ABC):
    """
    Abstract parent class for single turn safety teachers.

    Not meant to be a standalone teacher.
    """

    @classmethod
    def add_cmdline_args(
        cls, parser: ParlaiParser, partial_opt: Optional[Opt] = None
    ) -> ParlaiParser:
        super().add_cmdline_args(parser, partial_opt)
        parser = parser.add_argument_group('Safety Teacher Args')
        parser.add_argument(
            '--round',
            type=int,
            default=1,
            choices=[1, 2, 3],
            help='Which round of data to use',
        )
        parser.add_argument(
            '--round-only',
            type='bool',
            default=False,
            help='if False, includes all rounds up to including the specified '
            'round; if True, only includes data from the specified round',
        )
        return parser

    def __init__(self, opt, shared=None):
        build(opt['datapath'])  # download the data
        self.opt = opt
        self.data_path = os.path.join(
            opt['datapath'], 'dialogue_safety', SINGLE_TURN_DATA
        )

        self.fixed_random = random.Random(42)
        self.round = opt['round']
        self.label_candidates = [NOT_OK_CLASS, OK_CLASS]

        if shared and 'data' in shared:
            self.data = shared['data']
        else:
            self._setup_data(opt['datatype'])

        super().__init__(opt, shared)
        self.reset()

    @abstractmethod
    def _load_data_dump(self):
        pass

    def num_episodes(self):
        return len(self.data)

    def num_examples(self):
        return len(self.data)

    def _setup_data(self, datatype):
        # load data
        self.data_dump = self._load_data_dump()
        d_type = datatype.split(':')[0]
        self.data = []
        if not self.opt['round_only']:
            # loop and add other rounds
            for i in range(self.round - 1):
                rnd = str(i + 1)
                for x in ['good', 'bad']:
                    self.data += self.data_dump[d_type][rnd][x]

        # add data from current round
        for x in ['good', 'bad']:
            self.data += self.data_dump[d_type][str(self.round)][x]

        self.fixed_random.shuffle(self.data)

    def get(self, episode_idx, entry_idx):
        return Message(self.data[episode_idx])

    def share(self):
        shared = super().share()
        shared['data'] = self.data
        return shared



#!/usr/bin/env python3


# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

import parlai.core.build_data as build_data
import os
from parlai.core.build_data import DownloadableFile

RESOURCES = [
    DownloadableFile(
        'https://cims.nyu.edu/~sbowman/multinli/multinli_1.0.zip',
        'multinli_1.0.zip',
        '049f507b9e36b1fcb756cfd5aeb3b7a0cfcb84bf023793652987f7e7e0957822',
    )
]


def build(opt):
    dpath = os.path.join(opt['datapath'], 'MultiNLI')
    version = '1.0'

    if not build_data.built(dpath, version_string=version):
        print('[building data: ' + dpath + ']')

        if build_data.built(dpath):
            # an older version exists, so remove these outdated files.
            build_data.remove_dir(dpath)
        build_data.make_dir(dpath)

        # Download the data.
        for downloadable_file in RESOURCES:
            downloadable_file.download_file(dpath)

        # mark the data as built
        build_data.mark_done(dpath, version_string=version)



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

import json
import subprocess
from os.path import join as pjoin
from os.path import isfile
from os.path import isdir
from time import time
from parlai.core.params import ParlaiParser
from data_utils import word_url_tokenize, make_ccid_filter
from parlai.utils.io import PathManager

"""
Adapted from https://github.com/facebookresearch/ELI5/blob/master/data_creation/download_support_docs.py
to download specific CommonCrawl IDs and URLs.
"""


def make_docs_directory(output_dir, name):
    if not isdir(pjoin(output_dir, name)):
        subprocess.run(['mkdir', pjoin(output_dir, name)], stdout=subprocess.PIPE)
    for i in range(10):
        if not isdir(pjoin(output_dir, name, str(i))):
            subprocess.run(
                ['mkdir', pjoin(output_dir, name, str(i))], stdout=subprocess.PIPE
            )


def select_specific_ids(ccids):
    select = {}
    for i, ccid in enumerate(ccids):
        if not ccid.startswith('<urn:uuid:'):
            ccid = '<urn:uuid:' + ccid
        if not ccid.endswith('>'):
            ccid = ccid + '>'
        select[ccid] = ('specific_ids', i)
    return select


def select_specific_urls(urls):
    select = {}
    for i, url in enumerate(urls):
        if url.startswith('http://') or url.startswith('https://'):
            url = url.split('//', 1)[-1]
        select[url] = ('specific_urls', i)
    return select


def check_url(select, url):
    if url.startswith('http://') or url.startswith('https://'):
        url = url.split('//', 1)[-1]
    return select.get(url, False)


def setup_args():
    """
    Set up args.
    """
    parser = ParlaiParser(False, False)
    parser.add_parlai_data_path()
    cc = parser.add_argument_group('Download Support Docs')
    cc.add_argument(
        '-nw',
        '--slsize',
        default=716,
        type=int,
        metavar='N',
        help='number of wet files in a slice',
    )
    cc.add_argument(
        '-ns',
        '--slnum',
        default=0,
        type=int,
        metavar='N',
        help='commoncrawl slice number [0, ..., 71520 / args.slsize]',
    )
    cc.add_argument(
        '-wf',
        '--wet_urls',
        default='pre_computed/wet.paths',
        type=str,
        help='path from data folder to file containing WET file URLs',
    )
    cc.add_argument(
        '-sr_l',
        '--subreddit_names',
        default='["explainlikeimfive"]',
        type=str,
        help='subreddit names',
    )
    cc.add_argument(
        '-nu',
        '--n_urls',
        default=100,
        type=int,
        metavar='N',
        help='number of support documents to gather for each example',
    )
    cc.add_argument(
        '-sfq',
        '--save_freq',
        default=50,
        type=int,
        metavar='N',
        help='how often are results written to file',
    )
    cc.add_argument(
        '-o',
        '--output_dir',
        default='eli5',
        type=str,
        help='where to save the output in data folder',
    )
    cc.add_argument(
        '-u',
        '--urls',
        type=str,
        help='path to a json file of URLs to gather (in a list format)',
    )
    cc.add_argument(
        '-ids',
        '--ccuids',
        type=str,
        help='path to a json file of Common Crawl IDs to gather (in a list format)',
    )
    return parser.parse_args()


def main():
    opt = setup_args()
    output_dir = pjoin(
        opt['datapath'], opt['output_dir'], 'processed_data/collected_docs'
    )
    wet_urls_path = pjoin(opt['datapath'], opt['output_dir'], opt['wet_urls'])
    f = open(wet_urls_path, buffering=4096)
    url_lst = [line.strip() for line in f if line.strip() != '']
    f.close()
    if opt['urls']:
        with PathManager.open(opt['urls']) as f:
            specific_urls = json.load(f)
        using_specific_urls = True
        using_specific_ids = False
    elif opt['ccuids']:
        with PathManager.open(opt['ccuids']) as f:
            specific_ids = json.load(f)
        using_specific_urls = False
        using_specific_ids = True
    else:
        sr_names = json.loads(opt['subreddit_names'])
        using_specific_urls = False
        using_specific_ids = False
    print("loading URL selection")
    ccrawl_ids_maps = {}
    reddit_id_group = {}
    sr_names = json.loads(opt['subreddit_names'])
    # make a list of the CommonCrawl UIDs or URLs we want to process and keep
    if using_specific_urls:
        select_urls = select_specific_urls(specific_urls)
    elif using_specific_ids:
        select_ccid = select_specific_ids(specific_ids)
    else:
        for name in sr_names:
            print(name)
            ccrawl_ids_maps[name] = json.load(
                open('pre_computed/%s_ccrawl_ids.json' % (name,))
            )
            for i, (k, _) in enumerate(ccrawl_ids_maps[name]):
                reddit_id_group[k] = (i * 10) // len(ccrawl_ids_maps[name])
        select_ccid = make_ccid_filter(ccrawl_ids_maps, opt['n_urls'])

    print("loaded URL selection")
    # organize directories
    if not isdir(output_dir):
        subprocess.run(['mkdir', output_dir], stdout=subprocess.PIPE)
        if not isdir(pjoin(output_dir, 'tmp')):
            subprocess.run(['mkdir', pjoin(output_dir, 'tmp')], stdout=subprocess.PIPE)
    if using_specific_ids:
        make_docs_directory(output_dir, 'specific_ids')
    elif using_specific_urls:
        make_docs_directory(output_dir, 'specific_urls')
    else:
        for name in sr_names:
            make_docs_directory(output_dir, name)

    # check whether some ccrawl files have already been processed for this slice
    if using_specific_ids:
        articles = dict([('specific_ids', dict([(i, []) for i in range(10)]))])
        mode = 'ids'
    elif using_specific_urls:
        articles = dict([('specific_urls', dict([(i, []) for i in range(10)]))])
        mode = 'urls'
    else:
        articles = dict(
            [(name, dict([(i, []) for i in range(10)])) for name in sr_names]
        )
        mode = 'subreddits'
    # check progress of slice or if slice is finished
    if isfile(pjoin(output_dir, 'tmp', 'counts_%s_%d.json' % (mode, opt['slnum']))):
        start_line = json.load(
            open(pjoin(output_dir, 'tmp', 'counts_%s_%d.json' % (mode, opt['slnum'])))
        )
        if start_line == 'finished':
            return True
        for name in sr_names:
            for i_st in range(10):
                d_name = pjoin(output_dir, name, str(i_st))
                articles[name][i] = json.load(
                    open(pjoin(d_name, "docs_slice_%05d.json" % (opt['slnum'])))
                )
        print(
            "loaded previously downloaded pages:",
            start_line - opt['slnum'] * opt['slsize'],
        )
    else:
        start_line = opt['slnum'] * opt['slsize']
    # Download and parse slice of args.slsize WET files
    st_time = time()
    for i in range(start_line, min((opt['slnum'] + 1) * opt['slsize'], len(url_lst))):
        # Download wet file from amazon AWS
        dl_time = time()
        fname = url_lst[i].split('/')[-1][:-3]
        # download and unzip if necessary
        fpath = pjoin(output_dir, 'tmp', fname)
        print("processing", fpath)
        if not isfile(fpath):
            ct_try = 0
            while not isfile(fpath):
                subprocess.run(['rm', fpath + ".gz"], stdout=subprocess.PIPE)
                while not isfile(fpath + ".gz"):
                    url = "https://commoncrawl.s3.amazonaws.com/" + url_lst[i]
                    subprocess.run(
                        ['wget', '-P', pjoin(output_dir, 'tmp'), url],
                        stdout=subprocess.PIPE,
                    )
                    print("download:", time() - dl_time)
                    ct_try += 1
                    if ct_try > 5 and not isfile(fpath + ".gz"):
                        print("giving up on file", fname)
                        break
                downloaded = isfile(fpath + ".gz")
                if downloaded:
                    subprocess.run(['gunzip', fpath + ".gz"], stdout=subprocess.PIPE)
                    print("download and gunzip:", time() - dl_time)
                if ct_try > 5 and not isfile(fpath):
                    print("giving up on file", fname)
                    break
        else:
            downloaded = isfile(fpath)
        if not downloaded:
            print("FAILED DOWNLOADING ", fpath)
            continue
        # Extract, tokenize, and filter articles by language
        f = open(fpath, buffering=4096)
        article_url = ''
        article_id = ''
        article_txt = ''
        last_line = ''
        read_text = False
        ct = 0
        start_time = time()
        ccid_path_tuple = False
        # check and save pages by IDs if getting posts by IDs, or by URLs
        # if using URLs
        for line in f:
            if line.startswith("WARC/1.0"):
                if ccid_path_tuple:
                    ct += 1
                    article = {
                        'ccid': article_id,
                        'url': article_url,
                        'text': word_url_tokenize(article_txt),
                    }
                    if not using_specific_urls and not using_specific_ids:
                        name, eli_k, num = ccid_path_tuple
                        articles[name][reddit_id_group[eli_k]] += [
                            (eli_k, num, article)
                        ]
                    else:
                        name, num = ccid_path_tuple
                        articles[name][num % 10] += [(num, article)]
                article_txt = ''
                read_text = False
            if line.startswith("WARC-Target-URI"):
                try:
                    article_url = line.strip().split()[-1]
                    if using_specific_urls:
                        ccid_path_tuple = check_url(select_urls, article_url)
                except Exception:
                    article_url = '<UNK>'
                    if using_specific_urls:
                        ccid_path_tuple = False
            if line.startswith("WARC-Record-ID"):
                try:
                    article_id = line.strip().split()[-1]
                    if not using_specific_urls:
                        ccid_path_tuple = select_ccid.get(article_id, False)
                except Exception:
                    article_id = '<UNK>'
                    if not using_specific_urls:
                        ccid_path_tuple = False
            if read_text and (last_line.strip() + line.strip()) != '':
                article_txt += line + '\n'
                last_line = line
            if line.startswith("Content-Length: ") and ccid_path_tuple:
                read_text = True
        if ccid_path_tuple:
            ct += 1
            article = {
                'ccid': article_id,
                'url': article_url,
                'text': word_url_tokenize(article_txt),
            }

            if not using_specific_urls and not using_specific_ids:
                name, eli_k, num = ccid_path_tuple
                articles[name][reddit_id_group[eli_k]] += [(eli_k, num, article)]
            else:
                name, num = ccid_path_tuple
                articles[name][num % 10] += [(num, article)]
        f.close()
        subprocess.run(['rm', fpath], stdout=subprocess.PIPE)
        # periodically save slice
        print(">>>>>>>>>> ARTICLES FOUND %d in %.2f" % (ct, time() - start_time))
        if i % opt['save_freq'] == opt['save_freq'] - 1:
            for name, elik_maps in articles.items():
                print('saving', name, i, len(elik_maps))
                for i_st, ls in elik_maps.items():
                    d_name = pjoin(output_dir, name, str(i_st))
                    if not isdir(d_name):
                        subprocess.run(['mkdir', d_name], stdout=subprocess.PIPE)
                    json.dump(
                        ls,
                        open(
                            pjoin(d_name, "docs_slice_%05d.json" % (opt['slnum'])), 'w'
                        ),
                    )
            json.dump(
                i + 1,
                open(
                    pjoin(
                        output_dir, 'tmp', 'counts_%s_%d.json' % (mode, opt['slnum'])
                    ),
                    'w',
                ),
            )
            print('saved json files %.2f' % (time() - start_time,))
        subprocess.run(['rm', fpath], stdout=subprocess.PIPE)
    # save items to slices
    for name, elik_maps in articles.items():
        print('saving', name, i, len(elik_maps))
        for i_st, ls in elik_maps.items():
            d_name = pjoin(output_dir, name, str(i_st))
            if not isdir(d_name):
                subprocess.run(['mkdir', d_name], stdout=subprocess.PIPE)
            json.dump(
                ls, open(pjoin(d_name, "docs_slice_%05d.json" % (opt['slnum'])), 'w')
            )
    print('saved json files %.2f' % (time() - start_time,))
    json.dump(
        'finished',
        open(pjoin(output_dir, 'tmp', 'counts_%s_%d.json' % (mode, opt['slnum'])), 'w'),
    )
    print("processing slice %d took %f seconds" % (i, time() - st_time))


if __name__ == '__main__':
    main()



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.
from argparse import ArgumentParser
import json

from projects.self_feeding.utils import extract_fb_episodes, episode_to_examples
from parlai.utils.io import PathManager


def setup_args():
    parser = ArgumentParser()
    parser.add_argument(
        '-if', '--infile', type=str, default='data/ConvAI2/valid_self_original.txt'
    )
    parser.add_argument(
        '-of', '--outfile', type=str, default='data/convai2meta/valid.txt'
    )
    parser.add_argument(
        '-histsz',
        '--history-size',
        type=int,
        default=-1,
        help="The number of turns to include in the prompt."
        "In general, include all turns and filter in the teacher.",
    )
    config = vars(parser.parse_args())
    return config


def main(config):
    """
    Converts a Fbdialog file of episodes into a json file of Parley examples.
    """
    examples = []
    for episode in extract_fb_episodes(config['infile']):
        examples.extend(episode_to_examples(episode, config['history_size']))

    with PathManager.open(config['outfile'], 'w') as outfile:
        for ex in examples:
            outfile.write(json.dumps(ex.to_dict()) + '\n')


if __name__ == '__main__':
    config = setup_args()
    main(config)



#!/usr/bin/env python3


# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

import parlai.core.build_data as build_data
import os
import gzip
import json
from parlai.core.build_data import DownloadableFile
from parlai.utils.io import PathManager

RESOURCES = [
    DownloadableFile(
        'http://jmcauley.ucsd.edu/data/amazon/qa/qa_Appliances.json.gz',
        'qa_Appliances.json.gz',
        '9c613a5dfedd1071431faa29de903b1b0e592c5ac1c7861c26d8b69dfda8ac78',
        zipped=False,
    ),
    DownloadableFile(
        'http://jmcauley.ucsd.edu/data/amazon/qa/qa_Arts_Crafts_and_Sewing.json.gz',
        'qa_Arts_Crafts_and_Sewing.json.gz',
        'c9aad6d615294571c1be7ea6a88730829a68e701ca7d1168f4d6b5234c37ac65',
        zipped=False,
    ),
    DownloadableFile(
        'http://jmcauley.ucsd.edu/data/amazon/qa/qa_Automotive.json.gz',
        'qa_Automotive.json.gz',
        'ca2da4b9d3afd3e6c915d69b34618bdcf9c6febadd7389f368fb51e9e1585009',
        zipped=False,
    ),
    DownloadableFile(
        'http://jmcauley.ucsd.edu/data/amazon/qa/qa_Baby.json.gz',
        'qa_Baby.json.gz',
        'acca09d58e7b5487c41de1e21f5a4c676f9aaf1b0534cbeb73c2d5a7444231a0',
        zipped=False,
    ),
    DownloadableFile(
        'http://jmcauley.ucsd.edu/data/amazon/qa/qa_Beauty.json.gz',
        'qa_Beauty.json.gz',
        'e4e7d28e917fd1b0a605c85eff4c5d3e832a925ab6ed1216147b306984457b90',
        zipped=False,
    ),
    DownloadableFile(
        'http://jmcauley.ucsd.edu/data/amazon/qa/qa_Cell_Phones_and_Accessories.json.gz',
        'qa_Cell_Phones_and_Accessories.json.gz',
        '2ca990b05fbf1282aeeaab139ca8ad005cd540c0ad198ae2558d43cb01f24b42',
        zipped=False,
    ),
    DownloadableFile(
        'http://jmcauley.ucsd.edu/data/amazon/qa/qa_Clothing_Shoes_and_Jewelry.json.gz',
        'qa_Clothing_Shoes_and_Jewelry.json.gz',
        '2c8e24b6d89ccc4084c694a6197d862cdfdc90224877e566dc8170c86ba38d68',
        zipped=False,
    ),
    DownloadableFile(
        'http://jmcauley.ucsd.edu/data/amazon/qa/qa_Electronics.json.gz',
        'qa_Electronics.json.gz',
        '4ead1073043b6103a3a8eb738a04143f448cd3e200df074ad5071a24ad6bae95',
        zipped=False,
    ),
    DownloadableFile(
        'http://jmcauley.ucsd.edu/data/amazon/qa/qa_Grocery_and_Gourmet_Food.json.gz',
        'qa_Grocery_and_Gourmet_Food.json.gz',
        '438a9b97924098484c73d59e9f47e6bdb7898fdc94ae5fcfead0493a37443817',
        zipped=False,
    ),
    DownloadableFile(
        'http://jmcauley.ucsd.edu/data/amazon/qa/qa_Health_and_Personal_Care.json.gz',
        'qa_Health_and_Personal_Care.json.gz',
        '5f3564cf88a56fdf6913ca8e0e7e51478ef829b8faff6bec0e33f46e567bcbf6',
        zipped=False,
    ),
    DownloadableFile(
        'http://jmcauley.ucsd.edu/data/amazon/qa/qa_Home_and_Kitchen.json.gz',
        'qa_Home_and_Kitchen.json.gz',
        'eefa1f213bb476478c40ab9c2081c7dd0925f2f1df5d3efeb43f6e5715daaeaf',
        zipped=False,
    ),
    DownloadableFile(
        'http://jmcauley.ucsd.edu/data/amazon/qa/qa_Industrial_and_Scientific.json.gz',
        'qa_Industrial_and_Scientific.json.gz',
        '9ae1e9c2db859061350559d59c44125131d8af1149d6e99f45a73687f67a6b0e',
        zipped=False,
    ),
    DownloadableFile(
        'http://jmcauley.ucsd.edu/data/amazon/qa/qa_Musical_Instruments.json.gz',
        'qa_Musical_Instruments.json.gz',
        'a677605d6b6e612a485e939855f85c51a3bf1678da56c81ad5799ce1ef70d8bf',
        zipped=False,
    ),
    DownloadableFile(
        'http://jmcauley.ucsd.edu/data/amazon/qa/qa_Office_Products.json.gz',
        'qa_Office_Products.json.gz',
        '78e895f0df8e647d1c1a0a46f112f57adad9c49584dd48fb90e14648da790025',
        zipped=False,
    ),
    DownloadableFile(
        'http://jmcauley.ucsd.edu/data/amazon/qa/qa_Patio_Lawn_and_Garden.json.gz',
        'qa_Patio_Lawn_and_Garden.json.gz',
        '30a7bc60b39345a818a91a87f528afd860dc479d97115c4274056219471f5222',
        zipped=False,
    ),
    DownloadableFile(
        'http://jmcauley.ucsd.edu/data/amazon/qa/qa_Pet_Supplies.json.gz',
        'qa_Pet_Supplies.json.gz',
        '13a6a722f5f1c3209a0296ef87115ad2cc3ffa490b098c278596a0994484ef41',
        zipped=False,
    ),
    DownloadableFile(
        'http://jmcauley.ucsd.edu/data/amazon/qa/qa_Software.json.gz',
        'qa_Software.json.gz',
        '54847f45ed575611bc3abc88f176491678d1f0c38cad3a15ad5462f99c30e31c',
        zipped=False,
    ),
    DownloadableFile(
        'http://jmcauley.ucsd.edu/data/amazon/qa/qa_Sports_and_Outdoors.json.gz',
        'qa_Sports_and_Outdoors.json.gz',
        'a513460612695249027345f6cb8bde35d8dca664927582f4a1c429a45e60b62d',
        zipped=False,
    ),
    DownloadableFile(
        'http://jmcauley.ucsd.edu/data/amazon/qa/qa_Tools_and_Home_Improvement.json.gz',
        'qa_Tools_and_Home_Improvement.json.gz',
        'fe29b60e01c9867e7f62ea416bee300cd68f6c7230ac518f94f2fb252e322bc8',
        zipped=False,
    ),
    DownloadableFile(
        'http://jmcauley.ucsd.edu/data/amazon/qa/qa_Toys_and_Games.json.gz',
        'qa_Toys_and_Games.json.gz',
        '231582d4d5aa4959b6344d21ea0bf1b4872aa59645bb9e8107d1056922673dfe',
        zipped=False,
    ),
    DownloadableFile(
        'http://jmcauley.ucsd.edu/data/amazon/qa/qa_Video_Games.json.gz',
        'qa_Video_Games.json.gz',
        '45d626d00f393e9df0191d12e12fbdce6b6655ad14715bba2f8d9fe58fc7e1c2',
        zipped=False,
    ),
    DownloadableFile(
        'http://jmcauley.ucsd.edu/data/amazon/qa/icdm/QA_Automotive.json.gz',
        'QA2_Automotive.json.gz',
        '2a8209571736a56cfef5a3029545b614b3adfae1de166c179a2488f42fac2533',
        zipped=False,
    ),
    DownloadableFile(
        'http://jmcauley.ucsd.edu/data/amazon/qa/icdm/QA_Beauty.json.gz',
        'QA2_Beauty.json.gz',
        '1ac51134997c86cc59380b7f1be1885010dd1dd9e0aac7a503255c4f68bda7d0',
        zipped=False,
    ),
    DownloadableFile(
        'http://jmcauley.ucsd.edu/data/amazon/qa/icdm/QA_Cell_Phones_and_Accessories.json.gz',
        'QA2_Cell_Phones_and_Accessories.json.gz',
        '818a0cad91543a508f90ed0b3db11e71c964036a77c896a41cc7aa79283f1940',
        zipped=False,
    ),
    DownloadableFile(
        'http://jmcauley.ucsd.edu/data/amazon/qa/icdm/QA_Clothing_Shoes_and_Jewelry.json.gz',
        'QA2_Clothing_Shoes_and_Jewelry.json.gz',
        '47da83867a89fa7ab376040dc82944e74935cc81e1b86a6e3fc053a7a2dfdb32',
        zipped=False,
    ),
    DownloadableFile(
        'http://jmcauley.ucsd.edu/data/amazon/qa/icdm/QA_Electronics.json.gz',
        'QA2_Electronics.json.gz',
        'ab1081838b2f603708c3e6b9983a63f65512f46f1e1e0463f4437c88da0c7663',
        zipped=False,
    ),
    DownloadableFile(
        'http://jmcauley.ucsd.edu/data/amazon/qa/icdm/QA_Grocery_and_Gourmet_Food.json.gz',
        'QA2_Grocery_and_Gourmet_Food.json.gz',
        '140cdacf2b9bbb27bac0c60dc8a92f1c211cead68879451fac73c5e424bc1dd2',
        zipped=False,
    ),
    DownloadableFile(
        'http://jmcauley.ucsd.edu/data/amazon/qa/icdm/QA_Health_and_Personal_Care.json.gz',
        'QA2_Health_and_Personal_Care.json.gz',
        'c14dcd0bf837b3a26463de3cb2db8c3ad81bb262377ddc36610e8b674d49c833',
        zipped=False,
    ),
    DownloadableFile(
        'http://jmcauley.ucsd.edu/data/amazon/qa/icdm/QA_Home_and_Kitchen.json.gz',
        'QA2_Home_and_Kitchen.json.gz',
        '2822abbc06d24aa248c01b0a7edd982c8421c8eabaf5af421ded8eadbaf2dbfd',
        zipped=False,
    ),
    DownloadableFile(
        'http://jmcauley.ucsd.edu/data/amazon/qa/icdm/QA_Musical_Instruments.json.gz',
        'QA2_Musical_Instruments.json.gz',
        'e550e0f6a00db32966cb798ee6f3754cf8c16b474ae9ce3b0b8621faa45b4e15',
        zipped=False,
    ),
    DownloadableFile(
        'http://jmcauley.ucsd.edu/data/amazon/qa/icdm/QA_Office_Products.json.gz',
        'QA2_Office_Products.json.gz',
        '1ecb9cd8e7ad0f9049b2b0c350ce7eb304a32e685472da1d7da74dd8a5cb411f',
        zipped=False,
    ),
    DownloadableFile(
        'http://jmcauley.ucsd.edu/data/amazon/qa/icdm/QA_Patio_Lawn_and_Garden.json.gz',
        'QA2_Patio_Lawn_and_Garden.json.gz',
        'f78d127dbb9499af5f87ded005747061cc44c1114edeec8cae7346e044c697d3',
        zipped=False,
    ),
    DownloadableFile(
        'http://jmcauley.ucsd.edu/data/amazon/qa/icdm/QA_Pet_Supplies.json.gz',
        'QA2_Pet_Supplies.json.gz',
        'c0f31cb041a6bbc7dc5f9006c41458f8d9120c949fb6f1e6fe753cec4353e0f1',
        zipped=False,
    ),
    DownloadableFile(
        'http://jmcauley.ucsd.edu/data/amazon/qa/icdm/QA_Sports_and_Outdoors.json.gz',
        'QA2_Sports_and_Outdoors.json.gz',
        '521320c8162c17b361b070fe5f3fc8db4c4205508eff757f12fce284cbd555eb',
        zipped=False,
    ),
    DownloadableFile(
        'http://jmcauley.ucsd.edu/data/amazon/qa/icdm/QA_Tools_and_Home_Improvement.json.gz',
        'QA2_Tools_and_Home_Improvement.json.gz',
        '473ba7d623b8c43d832a9d7a3440796edcb4320211ca5b7c5dab69ed56b66a50',
        zipped=False,
    ),
    DownloadableFile(
        'http://jmcauley.ucsd.edu/data/amazon/qa/icdm/QA_Toys_and_Games.json.gz',
        'QA2_Toys_and_Games.json.gz',
        'dbeeaef459d266f383fea2845c8c76da6737812724abfb335db6a294870b1b10',
        zipped=False,
    ),
    DownloadableFile(
        'http://jmcauley.ucsd.edu/data/amazon/qa/icdm/QA_Video_Games.json.gz',
        'QA2_Video_Games.json.gz',
        'c54bb576b136146a3afa2a80033312b8b5ee6db721918db9875622e348a7c11c',
        zipped=False,
    ),
]


def parse_gzip(path):
    g = gzip.open(path, 'r')
    for l in g:
        yield json.dumps(eval(l))


def build(opt):
    dpath = os.path.join(opt['datapath'], 'AmazonQA')
    version = None

    if not build_data.built(dpath, version_string=version):
        print('[building data: ' + dpath + ']')

        if build_data.built(dpath):
            # an older version exists, so remove these outdated files.
            build_data.remove_dir(dpath)
        build_data.make_dir(dpath)

        # Download the data.
        for downloadable_file in RESOURCES:
            downloadable_file.download_file(dpath)
            new_filename = downloadable_file.file_name[:-3]
            print('[ unpacking data: ' + downloadable_file.file_name + ' ]')
            f = open(dpath + '/' + new_filename, 'w')
            for l in parse_gzip(dpath + '/' + downloadable_file.file_name):
                f.write(l + '\n')
            PathManager.rm(dpath + '/' + downloadable_file.file_name)

        # mark the data as built
        build_data.mark_done(dpath, version_string=version)



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.
#
# This task simply loads the specified file: useful for quick tests without
# setting up a new task.

from typing import Optional
from parlai.core.params import ParlaiParser
from parlai.core.opt import Opt
import copy
import os

from parlai.core.teachers import ConversationTeacher


class JsonTeacher(ConversationTeacher):
    """
    This module provides access to data in the Conversations format.

    See core/teachers.py for more info about the format.
    """

    @classmethod
    def add_cmdline_args(
        cls, parser: ParlaiParser, partial_opt: Optional[Opt] = None
    ) -> ParlaiParser:
        super().add_cmdline_args(parser, partial_opt)
        agent = parser.add_argument_group('JsonFile Task Arguments')
        agent.add_argument('-jfdp', '--jsonfile-datapath', type=str, help="Data file")
        agent.add_argument(
            '-jfdt',
            '--jsonfile-datatype-extension',
            type='bool',
            default=False,
            help="If true, use _train.jsonl, _valid.jsonl, _test.jsonl file extensions",
        )
        agent.add_argument(
            '--label-turns',
            type=str,
            help='which speaker to use as label',
            choices=['firstspeaker', 'secondspeaker', 'both'],
            default='secondspeaker',
        )
        return parser

    def __init__(self, opt, shared=None):
        super().__init__(opt, shared)
        opt = copy.deepcopy(opt)
        if not opt.get('jsonfile_datapath'):
            raise RuntimeError('jsonfile_datapath not specified')
        datafile = opt['jsonfile_datapath']
        if self.opt['jsonfile_datatype_extension']:
            datafile += "_" + self.opt['datatype'].split(':')[0] + '.jsonl'
        if shared is None:
            self._setup_data(datafile)
        # Truncate datafile to just the immediate enclosing folder name and file name
        dirname, basename = os.path.split(datafile)
        self.id = os.path.join(os.path.split(dirname)[1], basename)
        self.reset()


class DefaultTeacher(JsonTeacher):
    def __init__(self, opt, shared=None):
        super().__init__(opt, shared)



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.
from .build import download as download_dodeca


def download(datapath):
    download_dodeca(datapath, 'igc_ft')



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

from typing import Optional
from parlai.core.params import ParlaiParser
from parlai.core.opt import Opt
from copy import deepcopy
import torch.nn.functional as F

from parlai.core.worlds import DialogPartnerWorld, validate
import parlai.tasks.md_gender.utils as gend_utils


def get_threshold_preds(
    act, axis, self_threshold=-1, partner_threshold=-1, prettify=True
):
    pred = act['text']
    if prettify:
        pred = pred.split(':')[-1]

    if 'sorted_scores' in act:
        scores = F.softmax(act['sorted_scores'].float()).tolist()
        scores = [round(x, 4) for x in scores]
        cands = act['text_candidates']
        if prettify:
            cands = [
                cand.split(':')[-1].replace('gender-neutral', 'neutral')
                for cand in cands
            ]

        if axis == 'self' and self_threshold > 0:
            if scores[0] > self_threshold:
                pred = cands[0]
            else:
                pred = gend_utils.UNKNOWN
        elif axis == 'partner' and partner_threshold > 0:
            if cands[0] == 'neutral':
                pred = cands[0]
            else:  # male or female
                if scores[0] > partner_threshold:
                    pred = cands[0]
                else:
                    pred = gend_utils.UNKNOWN

        if prettify:
            strs = [f'{cand} ({score})' for cand, score in zip(cands, scores)]
            pred += '\n\t' + '[ ' + '\t'.join(strs) + ']'

    return pred


def get_axis_predictions(
    act_label, model_agent, self_threshold, partner_threshold, return_acts=False
):
    act = deepcopy(act_label)
    if 'labels' in act:
        del act['labels']
    if 'eval_labels' in act:
        del act['eval_labels']

    # SELF (as) pred
    act.force_set('label_candidates', gend_utils.SELF_CANDS)
    model_agent.observe(validate(act))
    self_act = model_agent.act()
    if not return_acts:
        self_pred = get_threshold_preds(
            self_act,
            axis='self',
            self_threshold=self_threshold,
            partner_threshold=partner_threshold,
        )
    else:
        self_pred = self_act

    # PARTNER (to) pred
    act.force_set('label_candidates', gend_utils.PARTNER_CANDS)
    model_agent.observe(validate(act))
    partner_act = model_agent.act()
    if not return_acts:
        partner_pred = get_threshold_preds(
            partner_act,
            axis='partner',
            self_threshold=self_threshold,
            partner_threshold=partner_threshold,
        )
    else:
        partner_pred = partner_act

    # ABOUT pred
    act.force_set('label_candidates', gend_utils.ABOUT_CANDS)
    model_agent.observe(validate(act))
    about_act = model_agent.act()
    if not return_acts:
        about_pred = get_threshold_preds(
            about_act,
            axis='about',
            self_threshold=self_threshold,
            partner_threshold=partner_threshold,
        )
    else:
        about_pred = about_act

    return self_pred, partner_pred, about_pred


class InteractiveWorld(DialogPartnerWorld):
    """
    Interact with a model and get TO/AS/ABOUT predictions for each utterance.
    """

    @classmethod
    def add_cmdline_args(
        cls, parser: ParlaiParser, partial_opt: Optional[Opt] = None
    ) -> ParlaiParser:
        super().add_cmdline_args(parser, partial_opt)
        parser = parser.add_argument_group('Gender Multiclass Interactive World')
        parser.add_argument(
            '--self-threshold',
            type=float,
            default=0.52,
            help='Threshold for choosing unknown for self',
        )
        parser.add_argument(
            '--partner-threshold',
            type=float,
            default=0.52,
            help='Threshold for choosing unknown for self',
        )
        parser.set_params(
            single_turn=True,  # this is a single turn task currently
            eval_candidates='inline',
            return_cand_scores=True,
        )
        return parser

    def __init__(self, opt, agents, shared=None):
        super().__init__(opt, agents, shared)

    def parley(self):
        """
        Agent 0 goes first.

        Alternate between the two agents.
        """
        acts = self.acts
        human_agent, model_agent = self.agents

        # human act
        act = deepcopy(human_agent.act())

        self_pred, partner_pred, about_pred = get_axis_predictions(
            act,
            model_agent,
            self_threshold=self.opt['self_threshold'],
            partner_threshold=self.opt['partner_threshold'],
        )

        pred_text = f'SELF: {self_pred}\nPARTNER: {partner_pred}\nABOUT: {about_pred}'
        acts[1] = {'id': 'MDGender Classifier', 'text': pred_text, 'episode_done': True}

        human_agent.observe(validate(acts[1]))
        self.update_counters()



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

"""
Tokenizer that is backed by spaCy (spacy.io).

Requires spaCy package and the spaCy english model.
"""

import spacy
import copy
from .tokenizer import Tokens, Tokenizer


class SpacyTokenizer(Tokenizer):
    def __init__(self, **kwargs):
        """
        Args:
            annotators: set that can include pos, lemma, and ner.
            model: spaCy model to use (either path, or keyword like 'en').
        """
        model = kwargs.get('model', 'en')
        self.annotators = copy.deepcopy(kwargs.get('annotators', set()))
        nlp_kwargs = {'parser': False}
        if not {'lemma', 'pos', 'ner'} & self.annotators:
            nlp_kwargs['tagger'] = False
        if not {'ner'} & self.annotators:
            nlp_kwargs['entity'] = False
        self.nlp = spacy.load(model, **nlp_kwargs)

    def tokenize(self, text):
        # We don't treat new lines as tokens.
        clean_text = text.replace('\n', ' ')
        tokens = self.nlp.tokenizer(clean_text)
        if {'lemma', 'pos', 'ner'} & self.annotators:
            self.nlp.tagger(tokens)
        if {'ner'} & self.annotators:
            self.nlp.entity(tokens)

        data = []
        for i in range(len(tokens)):
            # Get whitespace
            start_ws = tokens[i].idx
            if i + 1 < len(tokens):
                end_ws = tokens[i + 1].idx
            else:
                end_ws = tokens[i].idx + len(tokens[i].text)

            data.append(
                (
                    tokens[i].text,
                    text[start_ws:end_ws],
                    (tokens[i].idx, tokens[i].idx + len(tokens[i].text)),
                    tokens[i].tag_,
                    tokens[i].lemma_,
                    tokens[i].ent_type_,
                )
            )

        # Set special option for non-entity tag: '' vs 'O' in spaCy
        return Tokens(data, self.annotators, opts={'non_ent': ''})



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

from parlai.core.teachers import ParlAIDialogTeacher
from parlai.utils.misc import warn_once
from .build import build

import copy
import os


def _path(opt):
    # Build the data if it doesn't exist.
    build(opt)
    dt = opt['datatype'].split(':')[0]
    if dt == 'test':
        warn_once('WARNING: Test set not included. Setting datatype to valid.')
        dt = 'valid'
    return os.path.join(opt['datapath'], 'QuAC', dt + '.txt')


class DefaultTeacher(ParlAIDialogTeacher):
    def __init__(self, opt, shared=None):
        opt = copy.deepcopy(opt)
        opt['parlaidialogteacher_datafile'] = _path(opt)
        super().__init__(opt, shared)



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

"""
Throw a party.

## Examples

```shell
parlai party
```
"""

import sys
import time
from parlai.core.script import ParlaiScript, register_script
from parlai.core.params import ParlaiParser

DELAY = 75 / 1000
RESET = '\u001b[0m'
CLEAR_SCREEN = "\u001b[2J\u001b[H"
COLORS = [
    '\033[1;38;5;210m',  # peach
    '\033[1;38;5;222m',  # orange
    '\033[1;38;5;120m',  # green
    '\033[1;38;5;123m',  # cyan
    '\033[1;38;5;111m',  # blue
    '\033[1;38;5;134m',  # purple
    '\033[1;38;5;177m',  # pink
    '\033[1;38;5;207m',  # fuschia
    '\033[1;38;5;206m',  # magenta
    '\033[1;38;5;204m',  # red
]

# Frames from https://github.com/jmhobbs/terminal-parrot/blob/master/data.go,
# The MIT License (MIT)
#
# Copyright (c) 2016 John Hobbs
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


FRAMES = """
                       .cccc;;cc;';c.
                      .,:dkdc:;;:c:,:d:.
                     .loc'.,cc::::::,..,:.
                   .cl;....;dkdccc::,...c;
                  .c:,';:'..ckc',;::;....;c.
                .c:'.,dkkoc:ok:;llllc,,c,';:.
               .;c,';okkkkkkkk:,lllll,:kd;.;:,.
               co..:kkkkkkkkkk:;llllc':kkc..oNc
             .cl;.,okkkkkkkkkkc,:cll;,okkc'.cO;
             ;k:..ckkkkkkkkkkkl..,;,.;xkko:',l'
            .,...';dkkkkkkkkkkd;.....ckkkl'.cO;
         .,,:,.;oo:ckkkkkkkkkkkdoc;;cdkkkc..cd,
      .cclo;,ccdkkl;llccdkkkkkkkkkkkkkkkd,.c;
     .lol:;;okkkkkxooc::loodkkkkkkkkkkkko'.oc
   .c:'..lkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkd,.oc
  .lo;,ccdkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkd,.c;
,dx:..;lllllllllllllllllllllllllllllllloc'...
cNO;........................................`,
Z
                 .ckx;'........':c.
             .,:c:c:::oxxocoo::::,',.
            .odc'..:lkkoolllllo;..;d,
            ;c..:o:..;:..',;'.......;.
           ,c..:0Xx::o:.,cllc:,'::,.,c.
           ;c;lkXXXXXXl.;lllll;lXXOo;':c.
         ,dc.oXXXXXXXXl.,lllll;lXXXXx,c0:
         ;Oc.oXXXXXXXXo.':ll:;'oXXXXO;,l'
         'l;;OXXXXXXXXd'.'::'..dXXXXO;,l'
         'l;:0XXXXXXXX0x:...,:o0XXXXk,:x,
         'l;;kXXXXXXKXXXkol;oXXXXXXXO;oNc
        ,c'..ckk0XXXXXXXXXX00XXXXXXX0:;o:.
      .':;..:dd::ooooOXXXXXXXXXXXXXXXo..c;
    .',',:co0XX0kkkxx0XXXXXXXXXXXXXXX0c..;l.
  .:;'..oXXXXXXXXXXXXXXXXXXXXXXXXXXXXXko;';:.
.cdc..:oOXXXXXXXXKXXXXXXXXXXXXXXXXXXXXXXo..oc
:0o...:dxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxo,.:,
cNo........................................;'`,
Z
             .cc;.  ...  .;c.
         .,,cc:cc:lxxxl:ccc:;,.
        .lo;...lKKklllookl..cO;
      .cl;.,;'.okl;...'.;,..';:.
     .:o;;dkx,.ll..,cc::,..,'.;:,.
     co..lKKKkokl.':lllo;''ol..;dl.
   .,c;.,xKKKKKKo.':llll;.'oOxo,.cl,.
   cNo..lKKKKKKKo'';llll;;okKKKl..oNc
   cNo..lKKKKKKKko;':c:,'lKKKKKo'.oNc
   cNo..lKKKKKKKKKl.....'dKKKKKxc,l0:
   .c:'.lKKKKKKKKKk;....oKKKKKKo'.oNc
     ,:.,oxOKKKKKKKOxxxxOKKKKKKxc,;ol:.
     ;c..'':oookKKKKKKKKKKKKKKKKKk:.'clc.
   ,dl'.,oxo;'';oxOKKKKKKKKKKKKKKKOxxl::;,,.
  .dOc..lKKKkoooookKKKKKKKKKKKKKKKKKKKxl,;ol.
  cx,';okKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKl..;lc.
  co..:dddddddddddddddddddddddddddddddddl:;''::.
  co..........................................."`,
Z
            .ccccccc.
      .,,,;cooolccol;;,,.
     .dOx;..;lllll;..;xOd.
   .cdo,',loOXXXXXkll;';odc.
  ,oo:;c,':oko:cccccc,...ckl.
  ;c.;kXo..::..;c::'.......oc
,dc..oXX0kk0o.':lll;..cxxc.,ld,
kNo.'oXXXXXXo'':lll;..oXXOd;cOd.
KOc;oOXXXXXXo.':lol,..dXXXXl';xc
Ol,:k0XXXXXX0c.,clc'.:0XXXXx,.oc
KOc;dOXXXXXXXl..';'..lXXXXXd..oc
dNo..oXXXXXXXOx:..'lxOXXXXXk,.:; ..
cNo..lXXXXXXXXXOolkXXXXXXXXXkl;..;:.;.
.,;'.,dkkkkk0XXXXXXXXXXXXXXXXXOxxl;,;,;l:.
  ;c.;:''''':doOXXXXXXXXXXXXXXXXXXOdo;';clc.
  ;c.lOdood:'''oXXXXXXXXXXXXXXXXXXXXXk,..;ol.
  ';.:xxxxxocccoxxxxxxxxxxxxxxxxxxxxxxl::'.';;.
  ';........................................;l'`,
Z

        .;:;;,.,;;::,.
     .;':;........'co:.
   .clc;'':cllllc::,.':c.
  .lo;;o:coxdlooollc;',::,,.
.c:'.,cl,.'lc',,;;'......cO;
do;';oxoc::l;;llllc'.';;'.';.
c..ckkkkkkkd,;llllc'.:kkd;.':c.
'.,okkkkkkkkc;llllc,.:kkkdl,cO;
..;xkkkkkkkkc,ccll:,;okkkkk:,cl,
..,dkkkkkkkkc..,;,'ckkkkkkkc;ll.
..'okkkkkkkko,....'okkkkkkkc,:c.
c..ckkkkkkkkkdl;,:okkkkkkkkd,.',';.
d..':lxkkkkkkkkxxkkkkkkkkkkkdoc;,;'..'.,.
o...'';llllldkkkkkkkkkkkkkkkkkkdll;..'cdo.
o..,l;'''''';dkkkkkkkkkkkkkkkkkkkkdlc,..;lc.
o..;lc;;;;;;,,;clllllllllllllllllllllc'..,:c.
o..........................................;'`,
Z

           .,,,,,,,,,.
         .ckKxodooxOOdcc.
      .cclooc'....';;cool.
     .loc;;;;clllllc;;;;;:;,.
   .c:'.,okd;;cdo:::::cl,..oc
  .:o;';okkx;';;,';::;'....,;,.
  co..ckkkkkddk:,cclll;.,c:,:o:.
  co..ckkkkkkkk:,cllll;.:kkd,.':c.
.,:;.,okkkkkkkk:,cclll;.:kkkdl;;o:.
cNo..ckkkkkkkkko,.;llc,.ckkkkkc..oc
,dd;.:kkkkkkkkkx;..;:,.'lkkkkko,.:,
  ;c.ckkkkkkkkkkc.....;ldkkkkkk:.,'
,dc..'okkkkkkkkkxoc;;cxkkkkkkkkc..,;,.
kNo..':lllllldkkkkkkkkkkkkkkkkkdcc,.;l.
KOc,l;''''''';lldkkkkkkkkkkkkkkkkkc..;lc.
xx:':;;;;,.,,...,;;cllllllllllllllc;'.;oo,
cNo.....................................oc`,
Z


                   .ccccccc.
               .ccckNKOOOOkdcc.
            .;;cc:ccccccc:,::::,,.
         .c;:;.,cccllxOOOxlllc,;ol.
        .lkc,coxo:;oOOxooooooo;..:,
      .cdc.,dOOOc..cOd,.',,;'....':c.
      cNx'.lOOOOxlldOl..;lll;.....cO;
     ,do;,:dOOOOOOOOOl'':lll;..:d:.'c,
     co..lOOOOOOOOOOOl'':lll;.'lOd,.cd.
     co.,dOOOOOOOOOOOo,.;llc,.,dOOc..dc
     co..lOOOOOOOOOOOOc.';:,..cOOOl..oc
   .,:;.'::lxOOOOOOOOOo:'...,:oOOOc..dc
   ;Oc..cl'':llxOOOOOOOOdcclxOOOOx,.cd.
  .:;';lxl''''':lldOOOOOOOOOOOOOOc..oc
,dl,.'cooc:::,....,::coooooooooooc'.c:
cNo.................................oc`,
Z



                        .cccccccc.
                  .,,,;;cc:cccccc:;;,.
                .cdxo;..,::cccc::,..;l.
               ,oo:,,:c:cdxxdllll:;,';:,.
             .cl;.,oxxc'.,cc,.',;;'...oNc
             ;Oc..cxxxc'.,c;..;lll;...cO;
           .;;',:ldxxxdoldxc..;lll:'...'c,
           ;c..cxxxxkxxkxxxc'.;lll:'','.cdc.
         .c;.;odxxxxxxxxxxxd;.,cll;.,l:.'dNc
        .:,''ccoxkxxkxxxxxxx:..,:;'.:xc..oNc
      .lc,.'lc':dxxxkxxxxxxxdl,...',lx:..dNc
     .:,',coxoc;;ccccoxxxxxxxxo:::oxxo,.cdc.
  .;':;.'oxxxxxc''''';cccoxxxxxxxxxkxc..oc
,do:'..,:llllll:;;;;;;,..,;:lllllllll;..oc
cNo.....................................oc`,
Z


                              .ccccc.
                         .cc;'coooxkl;.
                     .:c:::c:,;,,,;c;;,.'.
                   .clc,',:,..:xxocc;...c;
                  .c:,';:ox:..:c,,,,,,...cd,
                .c:'.,oxxxxl::l:.;loll;..;ol.
                ;Oc..:xxxxxxxxx:.,llll,....oc
             .,;,',:loxxxxxxxxx:.,llll;.,;.'ld,
            .lo;..:xxxxxxxxxxxx:.'cllc,.:l:'cO;
           .:;...'cxxxxxxxxxxxxol;,::,..cdl;;l'
         .cl;':;'';oxxxxxxxxxxxxx:....,cooc,cO;
     .,,,::;,lxoc:,,:lxxxxxxxxxxxo:,,;lxxl;'oNc
   .cdxo;':lxxxxxxc'';cccccoxxxxxxxxxxxxo,.;lc.
  .loc'.'lxxxxxxxxocc;''''';ccoxxxxxxxxx:..oc
occ'..',:cccccccccccc:;;;;;;;;:ccccccccc,.'c,
Ol;......................................;l'`,
Z

                              ,ddoodd,
                         .cc' ,ooccoo,'cc.
                      .ccldo;....,,...;oxdc.
                   .,,:cc;.''..;lol;;,'..lkl.
                  .dkc';:ccl;..;dl,.''.....oc
                .,lc',cdddddlccld;.,;c::'..,cc:.
                cNo..:ddddddddddd;':clll;,c,';xc
               .lo;,clddddddddddd;':clll;:kc..;'
             .,:;..:ddddddddddddd:';clll;;ll,..
             ;Oc..';:ldddddddddddl,.,c:;';dd;..
           .''',:lc,'cdddddddddddo:,'...'cdd;..
         .cdc';lddd:';lddddddddddddd;.';lddl,..
      .,;::;,cdddddol;;lllllodddddddlcodddd:.'l,
     .dOc..,lddddddddlccc;'';cclddddddddddd;,ll.
   .coc,;::ldddddddddddddl:ccc:ldddddddddlc,ck;
,dl::,..,cccccccccccccccccccccccccccccccc:;':xx,
cNd.........................................;lOc
""".split(
    'Z'
)


@register_script('party', hidden=True, aliases=['parrot'])
class Party(ParlaiScript):
    @classmethod
    def setup_args(cls):
        parser = ParlaiParser(False, False, 'Throw a party!')
        parser.add_argument(
            '-n', '--seconds', default=-1, type=float, help='Number of seconds to party'
        )
        return parser

    def run(self):
        end = float('inf') if self.opt['seconds'] <= 0 else self.opt['seconds']
        i = 0
        timespent = 0
        while True:
            try:
                frame = FRAMES[i % len(FRAMES)]
                color = COLORS[i % len(COLORS)]
                i += 1
                sys.stdout.write(CLEAR_SCREEN)
                sys.stdout.write(color)
                sys.stdout.write('\n\n    ')
                sys.stdout.write(frame.replace('\n', '\n    '))
                sys.stdout.write(COLORS[(i * 3 + 1) % len(COLORS)])
                sys.stdout.write('\n\n              P A R T Y    P A R R O T\n')
                sys.stdout.write(RESET)
                time.sleep(DELAY)
                timespent += DELAY
                if timespent > end:
                    raise KeyboardInterrupt
            except KeyboardInterrupt:
                break
        sys.stdout.write(RESET + '\n')


if __name__ == '__main__':
    Party.main()



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

from parlai.core.script import superscript_main


def main():
    # indirect call so that console_entry (see setup.py) doesn't use the return
    # value of the final command as the return code. This lets us call
    # superscript_main as a function in other places (test_script.py).
    superscript_main()


if __name__ == '__main__':
    main()



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

from typing import Optional
from parlai.core.params import ParlaiParser
from parlai.core.opt import Opt
import copy
import json
import os

from parlai.utils.io import PathManager
from parlai.core.teachers import DialogTeacher

from .build import build

MULTINLI = 'MultiNLI'
MULTINLI_VERSION = '1.0'
MULTINLI_PREFIX = 'multinli_'
MULTINLI_PREMISE_PREFIX = 'Premise: '
MULTINLI_HYPO_PREFIX = 'Hypothesis: '
MULTINLI_LABELS = ['entailment', 'contradiction', 'neutral']
MULTINLI_PREMISE_KEY = 'sentence1'
MULTINLI_HYPO_KEY = 'sentence2'
MULTINLI_ANSWER_KEY = 'gold_label'
NOT_CONTRADICT = 'not_contradiction'
BICLASS_DICT = {
    'contradiction': 'contradiction',
    'entailment': NOT_CONTRADICT,
    'neutral': NOT_CONTRADICT,
}
BICLASS_LABELS = ['contradiction', NOT_CONTRADICT]


def _path(opt):
    build(opt)

    dt = opt['datatype'].split(':')[0]

    if dt == 'train':
        suffix = 'train'
    # Using matched set as valid and mismatched set as test
    elif dt == 'valid':
        suffix = 'dev_matched'
    elif dt == 'test':
        suffix = 'dev_mismatched'
    else:
        raise RuntimeError('Not valid datatype.')

    data_path = os.path.join(
        opt['datapath'],
        MULTINLI,
        MULTINLI_PREFIX + MULTINLI_VERSION,
        MULTINLI_PREFIX + MULTINLI_VERSION + '_' + suffix + '.jsonl',
    )
    return data_path


def setup_data(path, dialog_format=False, binary_classes=False):
    """
    Set up data in DialogData format from path.

    :param path: path to the data file that stores the MNLI dataset
    :param dialog_format: if set True, omit the special tokens 'Hypothesis' and 'Premise' in the text.
    :param binary_classes:  if set True, bucketize neutral and entailment in one (not_contradiction)
    :return:  a tuple in the parlai.core.teachers.DialogData format ``((x, y, r, c, i), new_episode?)`` where the ``x``
            is the query/question and ``y`` is the answer/label, ``clas`` represents the ``c`` the avaiable choices.
            ``new_episode`` is set True in any NLI teacher.
    """
    print('loading: ' + path)

    with PathManager.open(path, 'r') as data_file:
        for pair_line in data_file:
            pair = json.loads(pair_line)
            if pair[MULTINLI_ANSWER_KEY] == '-':
                continue

            question, answers, clas = convert_to_dialogData(
                premise_raw=pair[MULTINLI_PREMISE_KEY],
                hypo_raw=pair[MULTINLI_HYPO_KEY],
                answer_raw=pair[MULTINLI_ANSWER_KEY],
                dialog_format=dialog_format,
                binary_classes=binary_classes,
            )

            yield (question, answers, None, clas), True


def convert_to_dialogData(
    premise_raw, hypo_raw, answer_raw, dialog_format=False, binary_classes=False
):
    """
    Convert from NLI context to dialog text.

    :param premise_raw: raw premise extracted from jsonl file.
    :param hypo_raw: raw hypothesis extracted from jsonl file.
    :param answer_raw: raw answer extracted from jsonl file.
    :param dialog_format: if set True, omit the special tokens 'Hypothesis' and 'Premise' in the text.
    :param binary_classes: if set True, bucketize (neutral, entailment) into one (no_contradiction)
    :return: a tuple (question, answer, clas)
        - ``question`` (str) is a query and possibly context
        - ``answers`` (iter) is an iterable of label(s) for that query
        - ``clas`` (iter) is an iterable of label candidates that the student can choose from
    """
    premise_raw = premise_raw.strip('\n').strip('\t')
    hypo_raw = hypo_raw.strip('\n').strip('\t')
    clas = MULTINLI_LABELS

    if binary_classes:
        answer_raw = BICLASS_DICT[answer_raw]
        clas = BICLASS_LABELS
    if not dialog_format:
        premise_raw = MULTINLI_PREMISE_PREFIX + premise_raw
        hypo_raw = MULTINLI_HYPO_PREFIX + hypo_raw

    question = premise_raw + '\n' + hypo_raw
    answers = [answer_raw]

    return question, answers, clas


class DefaultTeacher(DialogTeacher):
    @classmethod
    def add_cmdline_args(
        cls, parser: ParlaiParser, partial_opt: Optional[Opt] = None
    ) -> ParlaiParser:
        super().add_cmdline_args(parser, partial_opt)
        parser = parser.add_argument_group('MNLI Teacher Args')
        parser.add_argument(
            '-dfm',
            '--dialog-format',
            type='bool',
            default=False,
            help="True if one would like to convert to a dialogue format without special tokens such as 'Premise'"
            " and 'Hypothesis' (default: False).",
        )
        parser.add_argument(
            '-bcl',
            '--binary-classes',
            type='bool',
            default=False,
            help="True if label candidates are (contradiction, not_contradiction), and (entailment, contradiction, "
            "neutral) otherwise (default: False).",
        )
        return parser

    def __init__(self, opt, shared=None):
        opt = copy.deepcopy(opt)
        data_path = _path(opt)
        opt['datafile'] = data_path
        self.id = 'MultiNLI'
        self.dialog_format = opt.get('dialog_format', False)
        self.binary_classes = opt.get('binary_classes', False)
        super().__init__(opt, shared)

    def setup_data(self, path):
        return setup_data(path, self.dialog_format, self.binary_classes)

    def label_candidates(self):
        if self.binary_classes:
            return BICLASS_LABELS
        return MULTINLI_LABELS



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.
"""
(A partial) implementation of the DrQa Document Reader from:

Danqi Chen, Adam Fisch, Jason Weston, Antoine Bordes. 2017.
Reading Wikipedia to Answer Open-Domain Questions.
In Association for Computational Linguistics (ACL).

Link: https://arxiv.org/abs/1704.00051

Note:
To use pretrained word embeddings, set the --embedding_file path argument.
GloVe is recommended, see http://nlp.stanford.edu/data/glove.840B.300d.zip.
To automatically download glove, use:
--embedding_file zoo:glove_vectors/glove.840B.300d.txt
"""

try:
    import torch
except ImportError:
    raise ImportError('Need to install pytorch: go to pytorch.org')

from typing import Optional
from parlai.core.params import ParlaiParser
from parlai.core.opt import Opt
import bisect
import numpy as np
import json
import random

from parlai.core.agents import Agent
from parlai.core.dict import DictionaryAgent
from parlai.core.build_data import modelzoo_path
from parlai.utils.io import PathManager
from . import config
from .utils import build_feature_dict, vectorize, batchify, normalize_text
from .model import DocReaderModel


# ------------------------------------------------------------------------------
# Dictionary.
# ------------------------------------------------------------------------------


class SimpleDictionaryAgent(DictionaryAgent):
    @classmethod
    def add_cmdline_args(
        cls, parser: ParlaiParser, partial_opt: Optional[Opt] = None
    ) -> ParlaiParser:
        group = super().add_cmdline_args(parser, partial_opt=partial_opt)
        group.add_argument(
            '--pretrained_words',
            type='bool',
            default=True,
            help='Use only words found in provided embedding_file',
        )
        return parser

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        # Index words in embedding file
        if (
            self.opt['pretrained_words']
            and self.opt.get('embedding_file')
            and not self.opt.get('trained', False)
        ):
            print('[ Indexing words with embeddings... ]')
            self.embedding_words = set()
            self.opt['embedding_file'] = modelzoo_path(
                self.opt.get('datapath'), self.opt['embedding_file']
            )
            with PathManager.open(self.opt['embedding_file']) as f:
                for line in f:
                    w = normalize_text(line.rstrip().split(' ')[0])
                    self.embedding_words.add(w)
            print('[ Num words in set = %d ]' % len(self.embedding_words))
        else:
            self.embedding_words = None

    def add_to_dict(self, tokens):
        """
        Builds dictionary from the list of provided tokens.

        Only adds words contained in self.embedding_words, if not None.
        """
        for token in tokens:
            if self.embedding_words is not None and token not in self.embedding_words:
                continue
            self.freq[token] += 1
            if token not in self.tok2ind:
                index = len(self.tok2ind)
                self.tok2ind[token] = index
                self.ind2tok[index] = token


# ------------------------------------------------------------------------------
# Document Reader.
# ------------------------------------------------------------------------------


class DrqaAgent(Agent):
    @classmethod
    def add_cmdline_args(
        cls, parser: ParlaiParser, partial_opt: Optional[Opt] = None
    ) -> ParlaiParser:
        # Runtime environment
        agent = parser.add_argument_group('DrQA Arguments')
        agent.add_argument('--no_cuda', type='bool', default=False)
        agent.add_argument('--gpu', type=int, default=-1)
        agent.add_argument('--random_seed', type=int, default=1013)

        # Basics
        agent.add_argument(
            '--embedding_file',
            type=str,
            default=None,
            help='file of space separated embeddings: w e1 ... ed',
        )
        agent.add_argument(
            '--init_model',
            type=str,
            default=None,
            help='Load dict/features/weights/opts from this file',
        )
        agent.add_argument('--log_file', type=str, default=None)

        # Model details
        agent.add_argument('--fix_embeddings', type='bool', default=True)
        agent.add_argument(
            '--tune_partial',
            type=int,
            default=0,
            help='Train the K most frequent word embeddings',
        )
        agent.add_argument(
            '--embedding_dim',
            type=int,
            default=300,
            help=('Default embedding size if ' 'embedding_file is not given'),
        )
        agent.add_argument(
            '--hidden_size', type=int, default=128, help='Hidden size of RNN units'
        )
        agent.add_argument(
            '--doc_layers', type=int, default=3, help='Number of RNN layers for passage'
        )
        agent.add_argument(
            '--question_layers',
            type=int,
            default=3,
            help='Number of RNN layers for question',
        )
        agent.add_argument(
            '--rnn_type',
            type=str,
            default='lstm',
            help='RNN type: lstm (default), gru, or rnn',
        )

        # Optimization details
        agent.add_argument(
            '--valid_metric',
            type=str,
            choices=['accuracy', 'f1'],
            default='f1',
            help='Metric for choosing best valid model',
        )
        agent.add_argument(
            '--max_len',
            type=int,
            default=15,
            help='The max span allowed during decoding',
        )
        agent.add_argument('--rnn_padding', type='bool', default=False)
        agent.add_argument(
            '--display_iter',
            type=int,
            default=10,
            help='Print train error after every' '<display_iter> epoches (default 10)',
        )
        agent.add_argument(
            '--dropout_emb',
            type=float,
            default=0.4,
            help='Dropout rate for word embeddings',
        )
        agent.add_argument(
            '--dropout_rnn', type=float, default=0.4, help='Dropout rate for RNN states'
        )
        agent.add_argument(
            '--dropout_rnn_output',
            type='bool',
            default=True,
            help='Whether to dropout the RNN output',
        )
        agent.add_argument(
            '--optimizer',
            type=str,
            default='adamax',
            help='Optimizer: sgd or adamax (default)',
        )
        agent.add_argument(
            '--learning_rate',
            '-lr',
            type=float,
            default=0.1,
            help='Learning rate for SGD (default 0.1)',
        )
        agent.add_argument(
            '--grad_clipping',
            type=float,
            default=10,
            help='Gradient clipping (default 10.0)',
        )
        agent.add_argument(
            '--weight_decay', type=float, default=0, help='Weight decay (default 0)'
        )
        agent.add_argument(
            '--momentum', type=float, default=0, help='Momentum (default 0)'
        )
        agent.add_argument(
            '--subsample-docs',
            type=int,
            default=0,
            help='When given paragraphs (separated by \n) will take '
            'only a subset of them, including the one with the '
            'answer for training.',
        )

        # Model-specific
        agent.add_argument('--concat_rnn_layers', type='bool', default=True)
        agent.add_argument(
            '--question_merge',
            type=str,
            default='self_attn',
            help='The way of computing question representation',
        )
        agent.add_argument(
            '--use_qemb',
            type='bool',
            default=True,
            help='Whether to use weighted question embeddings',
        )
        agent.add_argument(
            '--use_in_question',
            type='bool',
            default=True,
            help='Whether to use in_question features',
        )
        agent.add_argument(
            '--use_tf', type='bool', default=True, help='Whether to use tf features'
        )
        agent.add_argument(
            '--use_time',
            type=int,
            default=0,
            help='Time features marking how recent word was said',
        )
        cls.dictionary_class().add_cmdline_args(parser, partial_opt=partial_opt)
        return parser

    @staticmethod
    def dictionary_class():
        return SimpleDictionaryAgent

    def __init__(self, opt, shared=None):
        super().__init__(opt, shared)

        # All agents keep track of the episode (for multiple questions)
        self.episode_done = True

        self.opt['cuda'] = not self.opt['no_cuda'] and torch.cuda.is_available()

        if shared is not None:
            # model has already been set up
            self.word_dict = shared['word_dict']
            self.model = shared['model']
            self.feature_dict = shared['feature_dict']
        else:
            # set up model
            self.word_dict = DrqaAgent.dictionary_class()(opt)
            if self.opt.get('model_file') and PathManager.exists(opt['model_file']):
                self._init_from_saved(opt['model_file'])
            else:
                if self.opt.get('init_model'):
                    self._init_from_saved(opt['init_model'])
                else:
                    self._init_from_scratch()
            if self.opt['cuda']:
                print('[ Using CUDA (GPU %d) ]' % opt['gpu'])
                torch.cuda.set_device(opt['gpu'])
                self.model.cuda()

        # Set up params/logging/dicts
        self.id = self.__class__.__name__
        config.set_defaults(self.opt)
        self.n_examples = 0

    def _init_from_scratch(self):
        self.feature_dict = build_feature_dict(self.opt)
        self.opt['num_features'] = len(self.feature_dict)
        self.opt['vocab_size'] = len(self.word_dict)

        print('[ Initializing model from scratch ]')
        self.model = DocReaderModel(self.opt, self.word_dict, self.feature_dict)
        self.model.set_embeddings()

    def _init_from_saved(self, fname):
        print('[ Loading model %s ]' % fname)
        with PathManager.open(fname, 'rb') as f:
            saved_params = torch.load(f, map_location=lambda storage, loc: storage)
        if 'word_dict' in saved_params:
            # for compatibility with old saves
            self.word_dict.copy_dict(saved_params['word_dict'])
        self.feature_dict = saved_params['feature_dict']
        self.state_dict = saved_params['state_dict']
        config.override_args(self.opt, saved_params['config'])
        self.model = DocReaderModel(
            self.opt, self.word_dict, self.feature_dict, self.state_dict
        )

    def share(self):
        shared = super().share()
        shared['word_dict'] = self.word_dict
        shared['model'] = self.model
        shared['feature_dict'] = self.feature_dict
        return shared

    def observe(self, observation):
        # shallow copy observation (deep copy can be expensive)
        observation = observation.copy()
        if not self.episode_done and not observation.get('preprocessed', False):
            dialogue = self.observation['text'].split('\n')[:-1]
            dialogue.extend(observation['text'].split('\n'))
            observation['text'] = '\n'.join(dialogue)
        self.observation = observation
        self.episode_done = observation['episode_done']
        return observation

    def act(self):
        """Update or predict on a single example (batchsize = 1)."""
        reply = {'id': self.getID()}

        ex = self._build_ex(self.observation)
        if ex is None:
            return reply
        batch = batchify(
            [ex], null=self.word_dict[self.word_dict.null_token], cuda=self.opt['cuda']
        )

        # Either train or predict
        if 'labels' in self.observation:
            self.n_examples += 1
            self.model.update(batch)
        else:
            prediction, score = self.model.predict(batch)
            reply['text'] = prediction[0]
            reply['text_candidates'] = [prediction[0]]
            reply['candidate_scores'] = [score[0]]

        reply['metrics'] = {'train_loss': self.model.train_loss.avg}
        return reply

    def batch_act(self, observations):
        """
        Update or predict on a batch of examples.

        More efficient than act().
        """
        batchsize = len(observations)
        batch_reply = [{'id': self.getID()} for _ in range(batchsize)]

        # Some examples will be None (no answer found). Filter them.
        examples = [self._build_ex(obs) for obs in observations]
        valid_inds = [i for i in range(batchsize) if examples[i] is not None]
        examples = [ex for ex in examples if ex is not None]

        # If all examples are invalid, return an empty batch.
        if len(examples) == 0:
            return batch_reply

        # Else, use what we have (hopefully everything).
        batch = batchify(
            examples,
            null=self.word_dict[self.word_dict.null_token],
            cuda=self.opt['cuda'],
        )

        # Either train or predict
        if 'labels' in observations[0]:
            try:
                self.n_examples += len(examples)
                self.model.update(batch)
            except RuntimeError as e:
                # catch out of memory exceptions during fwd/bck (skip batch)
                if 'out of memory' in str(e):
                    print(
                        '| WARNING: ran out of memory, skipping batch. '
                        'if this happens frequently, decrease batchsize or '
                        'truncate the inputs to the model.'
                    )
                    batch_reply[0]['metrics'] = {'skipped_batches': 1}
                    return batch_reply
                else:
                    raise e

        else:
            predictions, scores = self.model.predict(batch)
            for i in range(len(predictions)):
                batch_reply[valid_inds[i]]['text'] = predictions[i]
                batch_reply[valid_inds[i]]['text_candidates'] = [predictions[i]]
                batch_reply[valid_inds[i]]['candidate_scores'] = [scores[i]]

        batch_reply[0]['metrics'] = {
            'train_loss': self.model.train_loss.avg * batchsize
        }
        return batch_reply

    def save(self, fname=None):
        """
        Save the parameters of the agent to a file.
        """
        fname = self.opt.get('model_file', None) if fname is None else fname
        if fname:
            print("[ saving model: " + fname + " ]")
            self.opt['trained'] = True
            self.model.save(fname)
            # save opt file
            with PathManager.open(fname + '.opt', 'w') as handle:
                json.dump(self.opt, handle)

    # --------------------------------------------------------------------------
    # Helper functions.
    # --------------------------------------------------------------------------

    def _build_ex(self, ex):
        """
        Find the token span of the answer in the context for this example.

        If a token span cannot be found, return None. Otherwise, torchify.
        """
        # Check if empty input (end of epoch)
        if 'text' not in ex:
            return

        # Split out document + question
        inputs = {}
        fields = ex['text'].strip().split('\n')

        # Data is expected to be text + '\n' + question
        if len(fields) < 2:
            raise RuntimeError('Invalid input. Is task a QA task?')

        paragraphs, question = fields[:-1], fields[-1]

        if len(fields) > 2 and self.opt.get('subsample_docs', 0) > 0 and 'labels' in ex:
            paragraphs = self._subsample_doc(
                paragraphs, ex['labels'], self.opt.get('subsample_docs', 0)
            )

        document = ' '.join(paragraphs)
        inputs['document'], doc_spans = self.word_dict.span_tokenize(document)
        inputs['question'] = self.word_dict.tokenize(question)
        inputs['target'] = None

        # Find targets (if labels provided).
        # Return if we were unable to find an answer.
        if 'labels' in ex:
            if 'answer_starts' in ex:
                # randomly sort labels and keep the first match
                labels_with_inds = list(zip(ex['labels'], ex['answer_starts']))
                random.shuffle(labels_with_inds)
                for ans, ch_idx in labels_with_inds:
                    # try to find an answer_start matching a tokenized answer
                    start_idx = bisect.bisect_left(
                        list(x[0] for x in doc_spans), ch_idx
                    )
                    end_idx = start_idx + len(self.word_dict.tokenize(ans)) - 1
                    if end_idx < len(doc_spans):
                        inputs['target'] = (start_idx, end_idx)
                        break
            else:
                inputs['target'] = self._find_target(inputs['document'], ex['labels'])
            if inputs['target'] is None:
                return

        # Vectorize.
        inputs = vectorize(self.opt, inputs, self.word_dict, self.feature_dict)

        # Return inputs with original text + spans (keep for prediction)
        return inputs + (document, doc_spans)

    def _find_target(self, document, labels):
        """
        Find the start/end token span for all labels in document.

        Return a random one for training.
        """

        def _positions(d, l):
            for i in range(len(d)):
                for j in range(i, min(len(d) - 1, i + len(l))):
                    if l == d[i : j + 1]:
                        yield (i, j)

        targets = []
        for label in labels:
            targets.extend(_positions(document, self.word_dict.tokenize(label)))
        if len(targets) == 0:
            return
        return targets[np.random.choice(len(targets))]

    def _subsample_doc(self, paras, labels, subsample):
        """
        Subsample paragraphs from the document (mostly for training speed).
        """
        # first find a valid paragraph (with a label)
        pi = -1
        for ind, p in enumerate(paras):
            for l in labels:
                if p.find(l):
                    pi = ind
                    break
        if pi == -1:
            # failed
            return paras[0:1]
        new_paras = []
        if pi > 0:
            for _i in range(min(subsample, pi - 1)):
                ind = random.randint(0, pi - 1)
                new_paras.append(paras[ind])
        new_paras.append(paras[pi])
        if pi < len(paras) - 1:
            for _i in range(min(subsample, len(paras) - 1 - pi)):
                ind = random.randint(pi + 1, len(paras) - 1)
                new_paras.append(paras[ind])
        return new_paras



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.
"""
Basic example which iterates through the tasks specified and runs the given model on
them.

## Examples

```shell
parlai display_model --task babi:task1k:1 --model repeat_label
parlai display_model --task convai2 --model-file "/path/to/model_file"  --datatype test
```
"""  # noqa: E501

from parlai.core.params import ParlaiParser
from parlai.core.agents import create_agent
from parlai.core.worlds import create_task
from parlai.core.script import ParlaiScript, register_script
from parlai.utils.strings import colorize
import parlai.utils.logging as logging

import random


def simple_display(opt, world, turn):
    if opt['batchsize'] > 1:
        raise RuntimeError('Simple view only support batchsize=1')
    teacher, response = world.get_acts()
    if turn == 0:
        text = "- - - NEW EPISODE: " + teacher.get('id', "[no agent id]") + "- - -"
        print(colorize(text, 'highlight'))
    text = teacher.get('text', '[no text field]')
    print(colorize(text, 'text'))
    response_text = response.get('text', 'No response')
    labels = teacher.get('labels', teacher.get('eval_labels', ['[no labels field]']))
    labels = '|'.join(labels)
    print(colorize('    labels: ' + labels, 'labels'))
    print(colorize('     model: ' + response_text, 'text2'))


def setup_args():
    parser = ParlaiParser(True, True, 'Display model predictions.')
    parser.add_argument('-n', '-ne', '--num-examples', default=10)
    parser.add_argument(
        '--display-add-fields',
        type=str,
        default='',
        help='Display these fields when verbose is off (e.g., "--display-add-fields label_candidates,beam_texts")',
    )
    # by default we want to display info about the validation set
    parser.set_defaults(datatype='valid')
    return parser


def display_model(opt):
    random.seed(42)

    # Create model and assign it to the specified task
    agent = create_agent(opt)
    world = create_task(opt, agent)
    agent.opt.log()

    # Show some example dialogs.
    turn = 0
    with world:
        for _k in range(int(opt['num_examples'])):
            world.parley()
            if opt['verbose'] or opt.get('display_add_fields', ''):
                print(world.display() + "\n~~")
            else:
                simple_display(opt, world, turn)
            turn += 1
            if world.get_acts()[0]['episode_done']:
                turn = 0
            if world.epoch_done():
                logging.info("epoch done")
                turn = 0
                break


@register_script('display_model', aliases=['dm'])
class DisplayModel(ParlaiScript):
    @classmethod
    def setup_args(cls):
        return setup_args()

    def run(self):
        display_model(self.opt)


if __name__ == '__main__':
    DisplayModel.main()



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.
"""
Pretrained LIGHT BERT bi-Ranker dialogue model from:

https://arxiv.org/pdf/1903.03094.pdf
"""

from parlai.core.build_data import download_models


def download(datapath):
    opt = {'datapath': datapath, 'model_type': 'biranker_dialogue'}  # for builder
    fnames = ['biranker_dialogue.tar.gz']
    download_models(opt, fnames, 'light', version='v0.5', use_model_type=True)



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

from parlai.scripts.interactive import setup_args, interactive

import random

if __name__ == '__main__':
    random.seed(42)
    parser = setup_args()

    parser.set_params(batchsize=1, beam_size=20, beam_min_n_best=10)

    print('\n' + '*' * 80)
    print('WARNING: This dialogue model is a research project that was trained on a')
    print(
        'large amount of open-domain Twitter data. It may generate offensive content.'
    )
    print('*' * 80 + '\n')

    interactive(parser.parse_args())



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

import random
import io
import os
import pickle
from parlai.utils.misc import msg_to_str

rand = random.Random(42)


def use_feat(opt, field, ttype):
    if 'all' in opt.get(field) or opt.get(field) == ttype:
        return True
    else:
        return False


mx = 0


def fix_labels(act, opt):
    labels = act.get('labels', act.get('eval_labels'))
    clip = int(opt.get('light_use_clip_cands', 1000))
    while len(act['label_candidates']) >= clip:
        act['label_candidates'].pop()
    is_label_cand = {}
    is_label_cand[labels] = False
    for c in act['label_candidates']:
        if c in is_label_cand:
            is_label_cand[c] = True
    for l, has in is_label_cand.items():
        if has is False:
            print("***ADDING A LABEL CAND****")
            act['label_candidates'].append(l)


def get_no_affordance_actions(in_room, carrying, other_carrying, other_name):
    all_actions = []
    # Drop "a" prefix from all
    in_room = [i[2:] for i in in_room]
    carrying = [i[2:] for i in carrying]
    other_carrying = [i[2:] for i in other_carrying]
    for obj in other_carrying:
        all_actions.append('steal {} from {}'.format(obj, other_name))
    for obj in in_room:
        all_actions.append('get {}'.format(obj))
    for obj in carrying:
        for f in ['drop {}', 'wield {}', 'wear {}', 'remove {}', 'eat {}', 'drink {}']:
            all_actions.append(f.format(obj))
        all_actions.append('give {} to {}'.format(obj, other_name))
        for obj2 in in_room:
            all_actions.append('put {} in {}'.format(obj, obj2))
        for obj2 in carrying:
            if obj2 == obj:
                continue
            all_actions.append('put {} in {}'.format(obj, obj2))
    return list(set(all_actions))


def gen_no_affordance_actions_for_dialogue(d):
    characters = [d['agents'][0]['name'], d['agents'][1]['name']]
    other_char = 1
    no_affordance_actions = []
    last_carry = []
    for idx in range(len(d['speech'])):
        char = characters[other_char]
        curr_carry = d['carrying'][idx] + d['wearing'][idx] + d['wielding'][idx]
        no_affordance_actions_turn = get_no_affordance_actions(
            d['room_objects'][idx], curr_carry, last_carry, char
        )
        no_affordance_actions_turn += d['available_actions'][idx]
        no_affordance_actions.append(list(set(no_affordance_actions_turn)))
        last_carry = curr_carry
        other_char = 1 - other_char
    d['no_affordance_actions'] = no_affordance_actions


def write_dialog(opt, fw, d, label_type, split):
    l = len(d['speech'])
    msgs = []
    text = ''

    d['which'] = []
    for i in range(0, len(d['emote'])):
        lab = 'none'
        if d['emote'][i] is not None:
            lab = 'emote'
        if d['action'][i] is not None:
            lab = 'action'
        d['which'].append(lab)

    if opt.get('light_use_taskname', True):
        text = '_task_' + label_type + '\n'
    if opt['light_use_setting']:
        text += (
            '_setting_name '
            + d['setting']['name']
            + ", "
            + d['setting']['category']
            + '\n'
        )
        text += '_setting_desc ' + d['setting']['description'] + '\n'
    if opt['light_use_person_names']:
        text += '_partner_name ' + d['partner_agent']['name'] + '\n'
        text += '_self_name ' + d['self_agent']['name'] + '\n'
    if use_feat(opt, 'light_use_persona', 'self'):
        text += '_self_persona ' + d['self_agent']['persona'] + '\n'
    if use_feat(opt, 'light_use_persona', 'other'):
        text += '_other_persona ' + d['partner_agent']['persona'] + '\n'
    if opt['light_use_objects']:
        for o, od in d['all_descriptions'].items():
            text += '_object_desc ' + o + " : " + od + '\n'
        if False:
            for o in d['room_objects'][0]:
                text += '_object_in_room ' + o + '\n'
            for o in d['carrying'][0]:
                text += '_object_carrying ' + o + '\n'
            for o in d['wearing'][0]:
                text += '_object_wearing ' + o + '\n'
            for o in d['wielding'][0]:
                text += '_object_wielding ' + o + '\n'
    for i in range(0, l, 2):
        if i < l - 1:
            if (
                use_feat(opt, 'light_use_speech', 'partner')
                and d['speech'][i] is not None
            ):
                if opt['light_use_speech_prefix']:
                    text += '_partner_say '
                text += str(d['speech'][i]) + '\n'
            if (
                use_feat(opt, 'light_use_action', 'partner')
                and d['action'][i] is not None
            ):
                text += '_partner_act ' + str(d['action'][i]) + '\n'
            if (
                use_feat(opt, 'light_use_emote', 'partner')
                and d['emote'][i] is not None
            ):
                text += '_partner_emote ' + str(d['emote'][i]) + '\n'
            if opt.get('light_use_repeat') == 'self_last':
                if i > 0:
                    text = str(d['speech'][i - 1])
                else:
                    text = 'nothing'
            if opt.get('light_use_repeat') == 'partner_last':
                text = str(d['speech'][i])
            if opt.get('light_use_repeat') == 'both_last':
                text = ''
                if i > 0:
                    text += str(d['speech'][i - 1]) + ' '
                text += str(d['speech'][i])

            label = d[label_type][i + 1]
            used_current = False
            shown = {}
            if (
                use_feat(opt, 'light_use_current_self_output', 'speech')
                and label_type != 'speech'
                and d['speech'][i + 1] is not None
            ):
                if 'remove' not in opt['light_use_current_self_output']:
                    if opt['light_use_speech_prefix']:
                        text += '_self_say '
                    text += str(d['speech'][i + 1]) + '\n'
                    shown['speech'] = True
                used_current = True
            if (
                use_feat(opt, 'light_use_current_self_output', 'all')
                and label_type != 'action'
                and use_feat(opt, 'light_use_action', 'self')
                and d['action'][i + 1] is not None
            ):
                if 'remove' not in opt['light_use_current_self_output']:
                    text += '_self_act ' + str(d['action'][i + 1]) + '\n'
                    shown['action'] = True
                used_current = True
            if (
                use_feat(opt, 'light_use_current_self_output', 'all')
                and label_type != 'emote'
                and use_feat(opt, 'light_use_emote', 'self')
                and d['emote'][i + 1] is not None
            ):
                if 'remove' not in opt['light_use_current_self_output']:
                    text += '_self_emote ' + str(d['emote'][i + 1]) + '\n'
                    shown['emote'] = True
                used_current = True
            if (
                'all_filtered' in opt['light_use_current_self_output']
                and used_current is False
            ):
                label = None
            if label is not None:
                msg = {}
                msg['text'] = text
                msg['labels'] = label
                add_negs(
                    msg,
                    d,
                    i + 1,
                    label_type,
                    split,
                    int(opt.get('light_use_cands', 100)),
                    opt.get('light_use_affordances', True),
                )
                msgs.append(msg)
                text = ''
            if (
                use_feat(opt, 'light_use_speech', 'self')
                and d['speech'][i + 1] is not None
                and ('speech' not in shown)
            ):
                if opt['light_use_speech_prefix']:
                    text += '_self_say '
                text += str(d['speech'][i + 1]) + '\n'
            if (
                use_feat(opt, 'light_use_action', 'self')
                and d['action'][i + 1] is not None
                and ('action' not in shown)
            ):
                text += '_self_act ' + str(d['action'][i + 1]) + '\n'
            if (
                use_feat(opt, 'light_use_emote', 'self')
                and d['emote'][i + 1] is not None
                and ('emote' not in shown)
            ):
                text += '_self_emote ' + str(d['emote'][i + 1]) + '\n'
    if len(msgs) > 0:
        msgs[-1]['episode_done'] = True
        for m in msgs:
            m['text'] = m['text'].rstrip('\n')
            # print(m.replace('\n', '\\n'))
            fix_labels(m, opt)
            global mx
            mx = max(len(m['label_candidates']), mx)
            # print(mx)
            txt = msg_to_str(m)
            fw.write(txt + '\n')


def write_alldata(opt, db, dpath, ltype, split):
    # for now train, valid and test will all be identical
    fname = os.path.join(dpath, ltype + "_" + split + ".txt")
    fw_tst = io.open(fname, 'w')
    for d in db:
        if d['split'] != split:
            continue
        d = d.copy()
        d['self_agent'] = d['agents'][1]
        d['partner_agent'] = d['agents'][0]
        write_dialog(opt, fw_tst, d, ltype, split)

        # now flip the conversation around so the target is the other speaker..
        d2 = d.copy()
        d2['self_agent'] = d2['agents'][0]
        d2['partner_agent'] = d2['agents'][1]
        d2['speech'] = list(d2['speech'])
        d2['speech'].insert(0, None)
        d2['emote'] = list(d2['emote'])
        d2['emote'].insert(0, None)
        d2['action'] = list(d2['action'])
        d2['action'].insert(0, None)
        d2['available_actions'] = list(d2['available_actions'])
        d2['available_actions'].insert(0, None)
        d2['no_affordance_actions'] = list(d2['no_affordance_actions'])
        d2['no_affordance_actions'].insert(0, None)
        write_dialog(opt, fw_tst, d2, ltype, split)
    fw_tst.close()


def add_negs(msg, d, ind, label_type, split, num_cands, use_affordances):
    if label_type == 'emote':
        msg['label_candidates'] = cands['emote']
    if label_type == 'which':
        msg['label_candidates'] = cands['which']
    if label_type == 'action':
        if use_affordances:
            msg['label_candidates'] = d['available_actions'][ind]
        else:
            msg['label_candidates'] = d['no_affordance_actions'][ind]
    if label_type == 'speech':
        cnt = 0
        label = msg['labels']
        negs = []
        # negs.append(label)
        used = {}
        # find 99 random negs that are != gold label
        while True:
            ind = rand.randint(0, len(cands['speech']) - 1)
            txt = cands['speech'][ind]
            if txt != label and ind not in used:
                negs.append(txt)
                used[ind] = True
                cnt += 1
                if cnt == num_cands - 1:
                    break
        # insert label in a random position
        negs.insert(rand.randrange(len(negs) + 1), label)
        msg['label_candidates'] = negs


cands = {}  # candidates


def write_out_candidates(db, dpath, dtype):
    emotes = {}
    acts = {}
    cands['emote'] = []
    cands['action'] = []
    cands['speech'] = []
    cands['which'] = ['none', 'action', 'emote']
    for d in db:
        gen_no_affordance_actions_for_dialogue(d)
        if d['split'] != dtype:
            continue
        for e in d['emote']:
            if e is not None and e not in emotes:
                cands['emote'].append(e)
                emotes[e] = True
        for act in d['action']:
            if act is not None and act not in acts:
                cands['action'].append(act)
                acts[act] = True
        for s in d['speech']:
            if s is not None:
                cands['speech'].append(s)
    for l in ['emote', 'speech', 'action', 'which']:
        fw = io.open(os.path.join(dpath, l + "_" + dtype + "_cands.txt"), 'w')
        for k in cands[l]:
            fw.write(k + "\n")
        fw.close


def build_from_db(opt, db_path, data_path, fname, fname2):
    # set up database
    dbp = os.path.join(db_path, fname)
    file = open(dbp, 'rb')
    db = pickle.load(file)
    # set up unseen database
    dbp = os.path.join(db_path, fname2)
    file = open(dbp, 'rb')
    db_unseen = pickle.load(file)

    # add test set labeling
    for i in range(0, len(db)):
        db[i]['split'] = 'train'
    for i in range(0, len(db_unseen)):
        db_unseen[i]['split'] = 'test_unseen'
    rand2 = random.Random(42)
    x = []
    # choose 1000 dialogues from the set that contain object manipulations
    # (hard-coded here).
    for i in range(1368, 4733):
        x.append(i)
    rand2.shuffle(x)
    for i in range(0, 1000):
        db[x[i]]['split'] = 'test'
    for i in range(1000, 1500):
        db[x[i]]['split'] = 'valid'

    for split in ['train', 'valid', 'test']:
        write_out_candidates(db, data_path, split)
        write_alldata(opt, db, data_path, 'speech', split)
        write_alldata(opt, db, data_path, 'action', split)
        write_alldata(opt, db, data_path, 'emote', split)
        write_alldata(opt, db, data_path, 'which', split)

    for split in ['test_unseen']:
        write_out_candidates(db_unseen, data_path, split)
        write_alldata(opt, db_unseen, data_path, 'speech', split)
        write_alldata(opt, db_unseen, data_path, 'action', split)
        write_alldata(opt, db_unseen, data_path, 'emote', split)
        write_alldata(opt, db_unseen, data_path, 'which', split)



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

"""
2.7B parameter Blender model distilled to 1.4B parameters (~2x inference speedup).

Please see <parl.ai/project/blender>.
"""

from .build import build

VERSION = 'v1.1'


def download(datapath):
    build(
        datapath,
        f'BST1Bdistill_{VERSION}.tgz',
        model_type='blender_1Bdistill',
        version=VERSION,
    )



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.
# Download and build the data if it does not exist.

import os
from tqdm import tqdm

import parlai.core.build_data as build_data
import parlai.utils.logging as logging

DATASET_NAME_LOCAL = 'NaturalQuestions'


def _import_google_cloud_client():
    try:
        from google.cloud import storage
    except ImportError:
        raise ImportError(
            'Please install Google Cloud Storage API:\n'
            '\tpip install --upgrade google-cloud-storage\n'
            'Or follow the instruction at '
            'https://cloud.google.com/storage/docs/reference/libraries'
        )
    return storage


def _download_with_cloud_storage_client(dpath):
    # Initiating the Cloud Storage Client with anonymous credentials
    stm = _import_google_cloud_client()
    storage_client = stm.Client.create_anonymous_client()

    def _download_blob(blob, target):
        with open(target, 'wb') as fout:
            storage_client.download_blob_to_file(blob, fout)

    def _download_blobs_from_list(blobs_list, target_path):
        for blob in tqdm(blobs_list):
            cloud_storage_fname = blob.name.split('/')[-1]
            downloaded_fname = os.path.join(target_path, cloud_storage_fname)
            _download_blob(blob, downloaded_fname)

    # Creating data storage directories
    for dtype in ('train', 'valid'):
        os.makedirs(os.path.join(dpath, dtype), exist_ok=True)

    # Populating a list of train and valid dataset blobs to download
    train_blobs = []
    valid_blobs = []
    for blob in storage_client.list_blobs('natural_questions'):
        blob_name = blob.name
        if not blob_name.endswith('.gz'):  # Not a zipped file
            continue

        if blob_name.startswith('v1.0/train'):
            train_blobs.append(blob)
        elif blob_name.startswith('v1.0/dev'):
            valid_blobs.append(blob)

    # Downloading the blobs to their respective dtype directory
    logging.info('Downloading train data ...')
    _download_blobs_from_list(train_blobs, os.path.join(dpath, 'train'))
    logging.info('Downloading valid data ...')
    _download_blobs_from_list(valid_blobs, os.path.join(dpath, 'valid'))


def _untar_dir_files(dtype_path):
    files = os.listdir(dtype_path)
    for fname in tqdm(files):
        build_data.ungzip(dtype_path, fname)


def _untar_dataset_files(dpath):
    for dtype in ('train', 'valid'):
        unzip_files_path = os.path.join(dpath, dtype)
        logging.info(f'Unzipping {dtype} files at {unzip_files_path}')
        _untar_dir_files(unzip_files_path)


def _move_valid_files_from_dev_to_valid(dpath):
    """
    Files from Google are stored at `nq-dev-##.jsonl.gz` and get untar'd to `nq-
    dev-##.jsonl`.

    The agent expects them to be stored at `nq-valid-00.jsonl`. This moves them over if
    need be.
    """
    valid_path = os.path.join(dpath, 'valid')
    for f in os.listdir(valid_path):
        if "dev" in f:
            new = f.replace('dev', 'valid')
            os.rename(os.path.join(valid_path, f), os.path.join(valid_path, new))


def build(opt):
    dpath = os.path.join(opt['datapath'], DATASET_NAME_LOCAL)
    version = 'v1.0'

    if not build_data.built(dpath, version_string=version):
        print('[building data: ' + dpath + ']')
        if build_data.built(dpath):
            # An older version exists, so remove these outdated files.
            build_data.remove_dir(dpath)
            logging.info('Removed the existing data (old version).')
        build_data.make_dir(dpath)
        _download_with_cloud_storage_client(dpath)
        _untar_dataset_files(dpath)
        _move_valid_files_from_dev_to_valid(dpath)
        build_data.mark_done(dpath, version_string=version)



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.
"""
Image+Seq2Seq Agent.
"""

from typing import Optional
from parlai.core.params import ParlaiParser
from parlai.core.opt import Opt
from typing import Dict, List, Tuple

import torch

from .modules import ImageSeq2seqModel, FusionType
from parlai.agents.transformer.transformer import TransformerGeneratorAgent
from parlai.core.dict import DictionaryAgent
from parlai.core.torch_agent import Batch
from parlai.core.torch_image_agent import TorchImageAgent


TOKEN_IMAGE = '__image__'
TOKEN_NO_IMAGE = '__no_image__'


class ImageSeq2seqAgent(TransformerGeneratorAgent, TorchImageAgent):
    """
    ImageSeq2seqAgent Agent.

    Combines a transformer generator with images.
    """

    def build_model(self) -> ImageSeq2seqModel:  # type: ignore
        """
        Override to build appropriate model.
        """
        self.model = ImageSeq2seqModel(self.opt, self.dict)
        if self.opt['embedding_type'] != 'random':
            self._copy_embeddings(
                self.model.embeddings.weight, self.opt['embedding_type']
            )
        return self.model

    @classmethod
    def add_cmdline_args(
        cls, parser: ParlaiParser, partial_opt: Optional[Opt] = None
    ) -> ParlaiParser:
        """
        Override to add one arg.
        """
        TransformerGeneratorAgent.add_cmdline_args(parser, partial_opt=partial_opt)
        TorchImageAgent.add_cmdline_args(parser, partial_opt=partial_opt)
        group = parser.add_argument_group('Image Encoder Args')
        group.add_argument(
            '--include-image-token',
            type='bool',
            default=True,
            recommended=True,
            help='if true, include image token (or no image token) for each example',
        )
        group.add_argument(
            '--image-fusion-type',
            type=str,
            default='late',
            choices=[f.value for f in FusionType],
            help='which fusion type to use',
        )
        return group

    def build_dictionary(self) -> DictionaryAgent:
        """
        Override to include image tokens.
        """
        self.dict = super().build_dictionary()
        if self.opt.get('include_image_token') and TOKEN_IMAGE not in self.dict:
            self.dict[TOKEN_IMAGE] = 1
            self.dict[TOKEN_NO_IMAGE] = 1

        return self.dict

    def _set_text_vec(self, *args, **kwargs) -> dict:
        """
        Override to include image token.
        """
        obs = super()._set_text_vec(*args, **kwargs)
        if 'text' not in obs or 'text_vec' not in obs:
            return obs
        if self.opt.get('include_image_token', False):
            # `truncate` is the third arg to this function
            truncate = args[2] - 1 if args[2] is not None else None
            vec = torch.LongTensor(  # type: ignore
                self._check_truncate(obs['text_vec'], truncate, True)
            )
            token = TOKEN_NO_IMAGE
            if obs.get('image', None) is not None:
                token = TOKEN_IMAGE
            obs.force_set(
                'text_vec',
                torch.cat([vec, vec.new_tensor(self.dict[token]).unsqueeze(0)], 0),
            )
        return obs

    def _dummy_batch(self, batchsize: int, maxlen: int) -> Batch:
        """
        Override to include image feats.
        """
        b = super()._dummy_batch(batchsize, maxlen)
        image = torch.ones(batchsize, self.image_features_dim).cuda()
        if self.n_image_channels > 1:
            image = image.unsqueeze(1).repeat(1, self.n_image_channels, 1)
        if self.fp16:
            image = image.half()
        return Batch(
            text_vec=b.text_vec,
            label_vec=b.label_vec,
            image=image,
            personalities=torch.ones(batchsize, self.opt['embedding_size']).cuda(),
        )

    def batchify_image_features(self, batch: Batch) -> Batch:
        """
        Format and return the batched image features.

        Image features represented by tensors will set to the right type.
        """
        if type(batch.image) == list and any(b is not None for b in batch.image):
            images = []
            for img in batch.image:
                if isinstance(img, torch.Tensor):
                    img = self._process_image_features(img)
                images.append(img)
            batch.image = images
        else:
            images = [None] * len(batch.valid_indices)
            batch.image = images
        return batch

    def _process_image_features(self, features: torch.Tensor) -> torch.Tensor:
        """
        Format shape and type of input image-feature tensor.

        Override TorchImageAgent._process_image_features to handle multi-dimensional
        images.
        """
        features = features.view(-1, self.image_features_dim)
        return torch.stack(
            [
                TorchImageAgent._process_image_features(self, features[i])
                for i in range(features.size(0))
            ]
        )

    def _model_input(self, batch: Batch) -> Tuple[torch.Tensor, List[object]]:
        return (batch.text_vec, batch.image)

    def load_state_dict(self, state_dict: Dict[str, torch.Tensor]):
        """
        Override for custom loading.

        Reasons:
            1. When using an init model without an image encoder
            2. We decide to add segment embeddings after the fact.
            3. When using an init model with only an encoder provided
                In this case, we may need to add the START token to the state_dict
            4. When using an init model without image tokens in the embeddings.
                This is only the case if the embs differ by 2 in dimension 0
        """
        state_dict['encoder.dummy_image_enc'] = self.model.encoder.dummy_image_enc
        state_dict['encoder.ones_mask'] = self.model.encoder.ones_mask
        # Case 1 -> No Image Encoder
        if 'encoder.image_encoder.0.weight' not in state_dict:
            for k, v in self.model.encoder.image_encoder.state_dict().items():
                state_dict[f'encoder.image_encoder.{k}'] = v

        # case 2 -> Segment embeddings in new model
        if (
            self.opt.get('n_segments', 0) >= 1
            and 'encoder.segment_embeddings.weight' not in state_dict
        ):
            state_dict[
                'encoder.segment_embeddings.weight'
            ] = self.model.encoder.segment_embeddings.weight

        # Case 3 -> Only an Encoder provided
        if not (any('decoder' in state_key for state_key in state_dict)):
            for k, v in self.model.decoder.state_dict().items():
                state_dict[f'decoder.{k}'] = v
            state_dict['decoder.embeddings.weight'] = state_dict['embeddings.weight']
            if 'START' not in state_dict:
                state_dict['START'] = self.model.START

        if self.opt['init_model'] is not None:
            try:
                self.model.load_state_dict(state_dict)
                return
            except RuntimeError as e:
                # Case 4 --> Check for Embedding Diffs. Make sure dims match up
                embs = state_dict['embeddings.weight']
                enc_embs = state_dict['encoder.embeddings.weight']
                dec_embs = state_dict['decoder.embeddings.weight']
                init_embs = self.model.embeddings.weight
                if (
                    embs.shape[0] + 2 != init_embs.shape[0]
                    or embs.shape[1] != init_embs.shape[1]
                ):
                    raise e

                state_dict.update(
                    {
                        'embeddings.weight': torch.cat(
                            (
                                embs.to(init_embs.device, dtype=init_embs.dtype),
                                init_embs[-2:, :],
                            )
                        ),
                        'encoder.embeddings.weight': torch.cat(
                            (
                                enc_embs.to(init_embs.device, dtype=init_embs.dtype),
                                init_embs[-2:, :],
                            )
                        ),
                        'decoder.embeddings.weight': torch.cat(
                            (
                                dec_embs.to(init_embs.device, dtype=init_embs.dtype),
                                init_embs[-2:, :],
                            )
                        ),
                    }
                )
                pct_init = round(embs.shape[0] / len(self.dict) * 100, 1)
                print(
                    f'Initialized embeddings for {embs.shape[0]} tokens ({pct_init}%)'
                )

        self.model.load_state_dict(state_dict)



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

import json
import os
import re
from datetime import datetime
from typing import Any, Dict

import numpy as np
import pandas as pd

from parlai.crowdsourcing.utils.acceptability import AcceptabilityChecker
from parlai.crowdsourcing.utils.analysis import AbstractTurnAnnotationResultsCompiler


class ModelChatResultsCompiler(AbstractTurnAnnotationResultsCompiler):
    """
    Compile and save results of human+model chats.

    Results will be saved on the level of specific conversations, as well as aggregated
    up the level of each worker as a whole.
    """

    @classmethod
    def setup_args(cls):
        parser = super().setup_args()
        parser.add_argument(
            '--start-date',
            type=str,
            default='',
            help='The earliest date to analyze results from',
        )
        parser.add_argument(
            '--max-convos-per-worker',
            type=int,
            default=100,
            help='The most conversations to analyze from any one user. Set to -1 for no limit.',
        )
        parser.add_argument(
            '--min-word-count',
            type=int,
            default=4,
            help='The minimum acceptable mean number of words per human utterance',
        )
        parser.add_argument(
            '--hit-block-list',
            type=str,
            default='',
            help='Comma-separated list of all hits to block',
        )
        parser.add_argument(
            '--worker-block-list',
            type=str,
            default='',
            help='Comma-separated list of all workers to block',
        )
        return parser

    def __init__(self, opt: Dict[str, Any]):

        super().__init__(opt)

        # Input args
        assert len(self.results_folders) > 0
        for folder in self.results_folders:
            assert os.path.isdir(folder), f'{folder} is not a valid folder!'
        os.makedirs(self.output_folder, exist_ok=True)
        self.start_date = opt['start_date']
        self.max_convos_per_worker = opt['max_convos_per_worker']
        self.min_word_count = opt['min_word_count']
        self.hit_block_list = opt['hit_block_list'].split(',')
        self.worker_block_list = opt['worker_block_list'].split(',')

        # Setting up problem buckets
        self.regular_buckets = [
            bucket
            for bucket in self.problem_buckets
            if bucket not in ['other', 'none_all_good']
        ]
        # Remove the buckets that are special cases

        self.acceptability_checker = AcceptabilityChecker()

    def get_results_path_base(self) -> str:
        now = datetime.now()
        return os.path.join(
            self.output_folder, f'results_{now.strftime("%Y%m%d_%H%M%S")}'
        )

    def compile_results(self) -> pd.DataFrame:

        read_folders = []
        date_strings = []
        for folder in self.results_folders:
            # Load paths
            date_strings = sorted(
                [
                    obj
                    for obj in os.listdir(folder)
                    if os.path.isdir(os.path.join(folder, obj))
                    and re.fullmatch(r'\d\d\d\d_\d\d_\d\d', obj)
                ]
            )
            if self.start_date != '':
                date_strings = [
                    str_ for str_ in date_strings if str_ >= self.start_date
                ]
            folders = [os.path.join(folder, str_) for str_ in date_strings]
            read_folders.extend(folders)
        print(f'Date folders: ' + ', '.join(date_strings))

        now = datetime.now()
        worker_results_file = os.path.join(
            self.output_folder, f'worker_results_{now.strftime("%Y%m%d_%H%M%S")}.csv'
        )
        # Read in each file
        num_incomplete_convos = 0
        num_complete_convos = 0
        complete_convos_per_model = {}
        bad_conversations = []
        problem_counts = {}
        worker_stats = {}
        worker_conversation_counts = {}
        total_utterances = 0

        conversation_idx = 0
        conversation_dfs = []
        for read_folder in read_folders:
            read_folder_name = os.path.split(read_folder)[-1]
            for file_name in sorted(os.listdir(read_folder)):
                if file_name in self.hit_block_list:
                    continue

                if 'incomplete' in file_name:
                    num_incomplete_convos += 1
                    continue
                else:
                    num_complete_convos += 1

                # Read in file
                with open(os.path.join(read_folder, file_name), 'rb') as f:
                    data = json.load(f)

                # Only include the first max_convos_per_worker conversations from a
                # worker to avoid biasing
                worker_id = data['workers'][0]
                assignment_id = data['assignment_ids'][0]
                if worker_id in worker_conversation_counts:
                    conversations_so_far = worker_conversation_counts[worker_id]
                else:
                    conversations_so_far = 0
                worker_conversation_counts[worker_id] = conversations_so_far + 1
                if (
                    self.max_convos_per_worker != -1
                    and conversations_so_far >= self.max_convos_per_worker
                ):
                    print(
                        f'Had {conversations_so_far} conversations already from this worker {worker_id}. Skipping {assignment_id}.'
                    )
                    continue

                # Check if need to block the turker
                word_counts = [
                    len(d['text'].split(' '))
                    for d in data['dialog']
                    if d['agent_idx'] == 0
                ]
                utterances = [d['text'] for d in data['dialog'] if d['agent_idx'] == 0]
                if np.average(word_counts) < self.min_word_count:
                    bad_conversations.append(data)
                    print(
                        f'Bad complete conversation, words from human: {utterances}. Skipping.'
                    )
                    continue

                if not all(
                    bucket in data['dialog'][1]['problem_data']
                    for bucket in self.problem_buckets
                ):
                    raise ValueError('Bucket(s) are missing from the problem data!')
                s = read_folder.split('/')[-2]
                experimental_design = s[s.find('_') + 1 :]

                model_nickname = experimental_design + '/' + data['workers'][1]
                if model_nickname not in problem_counts:
                    problem_counts[model_nickname] = {}
                if model_nickname in complete_convos_per_model:
                    complete_convos_per_model[model_nickname] += 1
                else:
                    complete_convos_per_model[model_nickname] = 1

                # Extract non-message info
                info_dict = {
                    'read_folder_name': read_folder_name,
                    'file_name': file_name,
                    'worker': worker_id,
                    'model_nickname': model_nickname,
                    'bad_workers': ','.join(data['bad_workers']),
                    'hit_id': data['hit_ids'][0],
                    'assignment_id': assignment_id,
                    'is_incomplete': 'incomplete' in file_name,
                    'context_dataset': data['context_dataset'],
                    'additional_context': data['additional_context'],
                }

                # Check that the conversation consists of pairs of comments between
                # agents 0 and 1, with 0 speaking first
                assert all(
                    [
                        utterance_data['agent_idx'] == utterance_idx % 2
                        for utterance_idx, utterance_data in enumerate(data['dialog'])
                    ]
                )

                # Determine whether the HIT contains unacceptable messages.
                # (We do this for every HIT, even if acceptability violation info
                # was already saved, because the violation criteria may have
                # changed since the HIT was collected.)
                messages_0 = [utt for utt in data['dialog'] if utt['agent_idx'] == 0]
                messages_1 = [utt for utt in data['dialog'] if utt['agent_idx'] == 1]
                assert len(messages_0) + len(messages_1) == len(data['dialog'])

                # Check the human utterances for safety
                utterances_0 = [m['text'] for m in messages_0]
                info_dict[
                    'acceptability_violations_0'
                ] = self.acceptability_checker.check_messages(
                    messages=utterances_0,
                    is_worker_0=True,
                    violation_types=self.acceptability_checker.ALL_VIOLATION_TYPES,
                )

                # Compile personas and previous utterances
                df = pd.DataFrame(
                    [],
                    columns=[
                        'folder',
                        'worker_id',
                        'hit_id',
                        'model_nickname',
                        'conversation_idx',
                        'turn_idx',
                        'agent_idx',
                        'text',
                    ]
                    + self.problem_buckets,
                )
                if (
                    data['additional_context'] is not None
                    and len(data['additional_context']) > 0
                ):
                    additional_context = '\n' + data['additional_context']
                else:
                    additional_context = ''
                df = df.append(
                    {
                        'folder': info_dict['read_folder_name'],
                        'worker_id': info_dict['worker'],
                        'hit_id': info_dict['hit_id'],
                        'model_nickname': model_nickname,
                        'conversation_idx': conversation_idx,
                        'turn_idx': -1,
                        'agent_idx': 1,
                        'text': 'your persona: '
                        + data['personas'][1][0]
                        + '\nyour persona: '
                        + data['personas'][1][1]
                        + additional_context,
                        **{bucket: '' for bucket in self.problem_buckets},
                    },
                    ignore_index=True,
                )

                total_utterances += len(
                    [d for d in data["dialog"] if d["agent_idx"] == 1]
                )
                if len(data['dialog']) > 20:
                    print(
                        f'Got long dialogue of {len(data["dialog"])} utterances, hit id: {info_dict["hit_id"]}, model_nickname: {model_nickname}.'
                    )

                dialog_has_problems = False
                for utterance_idx, utt in enumerate(data['dialog']):
                    d = {
                        'folder': info_dict['read_folder_name'],
                        'worker_id': info_dict['worker'],
                        'hit_id': info_dict['hit_id'],
                        'model_nickname': model_nickname,
                        'conversation_idx': conversation_idx,
                        'turn_idx': utterance_idx,
                        'agent_idx': utt['agent_idx'],
                        'text': utt['text'],
                        **{bucket: '' for bucket in self.problem_buckets},
                    }
                    if utt['agent_idx'] == 1:
                        if 'problem_data' not in utt:
                            for bucket in self.problem_buckets:
                                d[bucket] = 'MALFORMED'
                            print(
                                f'Warning got MALFORMED utterance problem data inside complete convo: {utt}. Skipping.'
                            )
                            continue
                        else:
                            for bucket in self.regular_buckets + ['none_all_good']:
                                d[bucket] = utt['problem_data'][bucket]
                            d['final_rating'] = (
                                utt['final_rating'] if 'final_rating' in utt else None
                            )
                        for k in self.regular_buckets + ['none_all_good']:
                            if k not in problem_counts[model_nickname]:
                                problem_counts[model_nickname][k] = 0
                            problem_counts[model_nickname][k] += d[k]
                            if k != 'none_all_good' and d[k]:
                                dialog_has_problems = True

                        if 'total' not in problem_counts[model_nickname]:
                            problem_counts[model_nickname]['total'] = 0
                        if d['agent_idx'] == 1:
                            problem_counts[model_nickname]['total'] += 1
                        if d['final_rating'] is not None:
                            # Only one the last utterance (agent idx == 1)
                            if 'count_ratings' not in problem_counts[model_nickname]:
                                problem_counts[model_nickname]['count_ratings'] = 0
                            problem_counts[model_nickname]['count_ratings'] += 1
                            if 'ratings' not in problem_counts[model_nickname]:
                                problem_counts[model_nickname]['ratings'] = []
                            problem_counts[model_nickname]['ratings'].append(
                                int(d['final_rating'])
                            )
                    else:
                        # Counting some aspects of the human's utterances
                        if (
                            'human_utterance_count'
                            not in problem_counts[model_nickname]
                        ):
                            problem_counts[model_nickname]['human_utterance_count'] = 0
                        problem_counts[model_nickname]['human_utterance_count'] += 1

                        if 'human_word_count' not in problem_counts[model_nickname]:
                            problem_counts[model_nickname]['human_word_count'] = 0
                        problem_counts[model_nickname]['human_word_count'] += len(
                            d['text'].strip().split(' ')
                        )

                        if 'human_question_count' not in problem_counts[model_nickname]:
                            problem_counts[model_nickname]['human_question_count'] = 0
                        problem_counts[model_nickname]['human_question_count'] += d[
                            'text'
                        ].count('?')
                    df = df.append(d, ignore_index=True)

                # Count the number of problems the worker got
                is_problem = ~df['none_all_good'].replace('', True)

                # Only want to count bot utterances but human ones, while included,
                # won't be False
                count = is_problem.sum()
                if info_dict['worker'] not in worker_stats:
                    worker_stats[info_dict['worker']] = {
                        'conversations': 0,
                        'problems_found': 0,
                    }
                worker_stats[info_dict['worker']]['conversations'] += 1
                worker_stats[info_dict['worker']]['problems_found'] += count

                # Logic for calculating percent of conversations that are clean
                if 'count_convos' not in problem_counts[model_nickname]:
                    problem_counts[model_nickname]['count_convos'] = 0
                problem_counts[model_nickname]['count_convos'] += 1

                if not dialog_has_problems:
                    if 'convo_clean' not in problem_counts[model_nickname]:
                        problem_counts[model_nickname]['convo_clean'] = 0
                    problem_counts[model_nickname]['convo_clean'] += 1

                # Adding the full conversation to the list of conversations
                conversation_dfs.append(df)
                conversation_idx += 1

        for m, conversation_count in complete_convos_per_model.items():
            print(f'Got {conversation_count} complete conversations for model: {m}')

        print(f'{num_complete_convos:d} complete conversations collected.')
        print(f'{len(bad_conversations):d} bad conversations.')
        num_approved_convos = num_complete_convos - len(bad_conversations)
        print(f'{num_approved_convos:d} approved conversations.')
        print(f'({num_incomplete_convos:d} incomplete conversations collected.)')
        for model_nickname, model_problems_dict in problem_counts.items():
            print(f'---{model_nickname}---')
            for p, v in model_problems_dict.items():
                if p == 'count_ratings':
                    continue
                if p == 'ratings':
                    print(
                        f'Average Engaging-ness Rating: {np.average(model_problems_dict["ratings"])} ({model_problems_dict["count_ratings"]} ratings)'
                    )
                    continue
                if p == 'human_word_count' or p == 'human_question_count':
                    print(
                        f'{p}: {v} ({v/model_problems_dict["human_utterance_count"]:.3})'
                    )
                elif p == 'human_utterance_count':
                    print(f'{p}: {v}')
                elif p == 'count_convos':
                    print(f'{p}: {v}')
                elif p == 'convo_clean':
                    print(f'{p}: {v} ({v/model_problems_dict["count_convos"]:.2%})')
                else:
                    print(f'{p}: {v} ({v/model_problems_dict["total"]:.2%})')

        print('Printing worker IDs not already in block list to add...')
        for b in bad_conversations:
            worker_id = b['workers'][0]
            if worker_id not in self.worker_block_list:
                print(f"""'{worker_id}',""")
        print('Done printing bad workers.')

        print('Worker stats:')
        worker_df = pd.DataFrame(
            [],
            columns=[
                'worker_id',
                'conversations',
                'problems_found',
                'avg_problems_per_convo',
            ],
        )

        for worker_id, data in worker_stats.items():
            print(worker_id)
            avg_problems_per_convo = data['problems_found'] / data['conversations']
            stat = {
                'worker_id': worker_id,
                'conversations': data['conversations'],
                'problems_found': data['problems_found'],
                'avg_problems_per_convo': avg_problems_per_convo,
            }
            worker_df = worker_df.append(stat, ignore_index=True)
        worker_df = worker_df.sort_values('avg_problems_per_convo', ascending=0)
        worker_df.to_csv(worker_results_file, index=False)
        print(worker_df)
        print(f'Wrote worker statistical results to: {worker_results_file}')

        html_text = (
            '<html><body><table><tr><td>model</td>'
            + ''.join(f'<td>{bucket}</td>' for bucket in self.regular_buckets)
            + '<td>none_all_good</td><td>human_word_count</td><td>human_question_count</td><td>convo_clean</td><td>final_rating</td></tr>'
        )
        for model_nickname, model_problems_dict in problem_counts.items():
            html_text += f'<tr><td>{model_nickname}</td>'
            keys = self.regular_buckets + [
                'none_all_good',
                'human_word_count',
                'human_question_count',
            ]
            for k in keys:
                if k == 'human_word_count' or k == 'human_question_count':
                    html_text += f'<td>{model_problems_dict[k]} ({model_problems_dict[k]/model_problems_dict["human_utterance_count"]:.3} average)</td>'
                else:
                    html_text += f'<td>{model_problems_dict[k]} ({model_problems_dict[k]/model_problems_dict["total"]:.1%})</td>'
            html_text += f'<td>{model_problems_dict["convo_clean"]/model_problems_dict["count_convos"]:.1%} ({model_problems_dict["count_convos"]} convos)</td>'

            html_text += f'<td>{np.average(model_problems_dict["ratings"]):.2f} ({model_problems_dict["count_ratings"]} ratings)</td>'

            html_text += '</tr>'

        html_text += '</table>'
        print(html_text)

        # Save full results
        all_conversations_df = pd.DataFrame()
        for df in conversation_dfs:
            all_conversations_df = all_conversations_df.append(df)
        print(f'\nWorker conversation counts: {worker_conversation_counts}')

        return all_conversations_df


if __name__ == '__main__':
    parser_ = ModelChatResultsCompiler.setup_args()
    args_ = parser_.parse_args()
    ModelChatResultsCompiler(vars(args_)).compile_and_save_results()



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.
#
# Download and build the data if it does not exist.

import parlai.core.build_data as build_data
import os
from parlai.core.build_data import DownloadableFile

RESOURCES = [
    DownloadableFile(
        '1u5zzfENGbRYVo-HsyFXZc3sJ9FgDTNx4',
        'raw_train_data.json',
        '7380e41ca8c65084140af997057eb9e8f974e08a19fdb40de73a9f96e4b5bd6d',
        from_google=True,
        zipped=False,
    ),
    DownloadableFile(
        '1nRsAyuVZu7L2f2YcxNbxsT1gZzFnQy-P',
        'raw_test_data.json',
        '3fd2cc672fbae118f3545640fc4c4f45a2e9037c98eebd1e64ad2e0ce5d1fe35',
        from_google=True,
        zipped=False,
    ),
]


def build(opt):
    dpath = os.path.join(opt['datapath'], 'holl_e')
    version = None

    if not build_data.built(dpath, version_string=version):
        print('[building data: ' + dpath + ']')
        if build_data.built(dpath):
            # An older version exists, so remove these outdated files.
            build_data.remove_dir(dpath)
        build_data.make_dir(dpath)

        # Download the data.
        for downloadable_file in RESOURCES:
            downloadable_file.download_file(dpath)

        # Mark the data as built.
        build_data.mark_done(dpath, version_string=version)



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

from parlai.core.teachers import FbDeprecatedDialogTeacher
from .build import build

import copy
import os


def _path(task, opt, dt):
    # Build the data if it doesn't exist.
    build(opt)
    return os.path.join(
        opt['datapath'], 'wmt', '{task}_{type}.txt'.format(task=task, type=dt)
    )


class EnDeTeacher(FbDeprecatedDialogTeacher):
    def __init__(self, opt, shared=None):
        opt = copy.deepcopy(opt)
        self.task_name = 'en_de'
        dt = opt['datatype'].split(':')[0]
        opt['datafile'] = _path(self.task_name, opt, dt)
        super().__init__(opt, shared)


class DefaultTeacher(EnDeTeacher):
    pass



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

import copy
import json
import os

from parlai.utils.io import PathManager
from parlai.core.teachers import DialogTeacher

from .build import build, DECODE, DECODE_PREFIX, DECODE_VERSION


def _path(opt, test_type):
    build(opt)

    dt = opt['datatype'].split(':')[0]

    if dt == 'train':
        suffix = 'train'
    elif dt == 'valid':
        suffix = 'dev'
    elif dt == 'test':
        if test_type == 'vanilla':
            suffix = "test"
        else:
            suffix = test_type  # human-bot, act, or rct
    else:
        raise RuntimeError('Not valid datatype.')

    data_directory = os.path.join(
        opt['datapath'], DECODE, DECODE_PREFIX + DECODE_VERSION
    )

    data_path = os.path.join(data_directory, suffix + '.jsonl')

    return data_path


class DecodeTeacher(DialogTeacher):
    @classmethod
    def add_cmdline_args(cls, parser, partial_opt):
        super().add_cmdline_args(parser, partial_opt)
        parser = parser.add_argument_group('DECODE Teacher Args')
        parser.add_argument(
            '--test_type',
            type=str,
            default='vanilla',
            help="The test sets can have three types: vanilla (default), human-bot, a2t, and rct.",
        )
        return parser

    def __init__(self, opt, shared=None):
        opt = copy.deepcopy(opt)
        self.datatype = opt['datatype']
        test_type = opt.get('test_type', "vanilla")

        data_path = _path(opt, test_type)
        opt['datafile'] = data_path

        self.id = 'decode'
        # this can pass in other arguments.
        # self.dialog_format = opt.get('dialog_format', False)
        super().__init__(opt, shared)

    def setup_data(self, path):
        # note that path is the value provided by opt['datafile']
        print('loading: ' + path)
        with PathManager.open(path) as data_file:
            for data_line in data_file:
                data_item = json.loads(data_line)

                for turn_id, turn in enumerate(data_item['turns']):
                    new_episode = False
                    turn['labels'] = "none"
                    del turn['auxiliary']
                    if turn_id == 0:
                        new_episode = True
                    if turn_id == len(data_item['turns']) - 1:
                        if data_item['is_contradiction']:
                            turn['labels'] = 'contradiction'
                        else:
                            turn['labels'] = 'non_contradiction'
                        turn['auxiliary'] = {
                            "contradiction_indices": data_item[
                                'aggregated_contradiction_indices'
                            ]
                        }

                    yield turn, new_episode


class DefaultTeacher(DecodeTeacher):
    pass



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

import numpy as np
from parlai.core.script import ParlaiScript, register_script
from parlai.core.agents import create_agent
from parlai.core.torch_agent import TorchAgent
from parlai.core.worlds import create_task
from parlai.core.params import ParlaiParser
from parlai.utils.misc import TimeLogger, nice_report
import parlai.utils.logging as logging


@register_script("token_stats", hidden=True)
class TokenStats(ParlaiScript):
    @classmethod
    def setup_args(cls):
        parser = ParlaiParser(True, True, description='Compute tokenized stats.')
        parser.add_argument('--num-examples', '-n', type=int, default=-1)
        parser.add_argument('-ltim', '--log-every-n-secs', type=float, default=10)
        parser.add_argument('--field', default='text')
        parser.add_argument('--final-only', type='bool', default=False)
        parser.set_defaults(
            no_cuda=True, model='test_agents/null', datatype='train:stream:ordered'
        )
        return parser

    def _compute_stats(self, lengths):
        lengths = np.array(lengths)
        return {
            "exs": len(lengths),
            "min": np.min(lengths),
            "max": np.max(lengths),
            "mean": np.median(lengths),
            "p01": np.quantile(lengths, 0.05),
            "p05": np.quantile(lengths, 0.05),
            "p10": np.quantile(lengths, 0.10),
            "p25": np.quantile(lengths, 0.25),
            "p50": np.quantile(lengths, 0.50),
            "p75": np.quantile(lengths, 0.75),
            "p90": np.quantile(lengths, 0.90),
            "p95": np.quantile(lengths, 0.95),
            "p99": np.quantile(lengths, 0.99),
            "p@128": np.mean(lengths <= 128),
        }

    def run(self):
        self.opt['no_cuda'] = True
        if 'ordered' not in self.opt['datatype'] and 'train' in self.opt['datatype']:
            self.opt['datatype'] = self.opt['datatype'] + ':ordered'
        agent = create_agent(self.opt)
        agent.opt.log()
        num_examples = self.opt['num_examples']
        field = self.opt['field'] + '_vec'
        if num_examples < 0:
            num_examples = float('inf')
        assert self.opt['batchsize'] == 1
        assert isinstance(agent, TorchAgent)

        world = create_task(self.opt, agent)
        teacher = world.get_task_agent()

        # set up logging
        log_every_n_secs = self.opt.get('log_every_n_secs', -1)
        if log_every_n_secs <= 0:
            log_every_n_secs = float('inf')
        log_time = TimeLogger()

        lengths = []

        cnt = 0
        total = min(teacher.num_examples(), num_examples)
        while not teacher.epoch_done() and cnt < num_examples:
            act = teacher.act()
            processed = agent.observe(act)
            try:
                text_vec = processed[field]
            except KeyError:
                raise KeyError(f"Pick one of {list(processed.keys())}")
            if text_vec is not None and (
                not self.opt['final_only'] or act.get('episode_done')
            ):
                cnt += 1
                lengths.append(float(len(text_vec)))
            agent.self_observe({})

            if log_time.time() > log_every_n_secs:
                report = self._compute_stats(lengths)
                text, report = log_time.log(report['exs'], total, report)
                logging.info(text)

        report = self._compute_stats(lengths)
        print(nice_report(report))
        return report


if __name__ == '__main__':
    TokenStats.main()



#!/usr/bin/env python3

# Copyright (c) Facebook, Inc. and its affiliates.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

from parlai.core.image_featurizers import ImageLoader
from parlai.core.metrics import TeacherMetrics
from parlai.core.teachers import Teacher
from parlai.utils.io import PathManager
from .build import build

import json
import random
import os


def _path(opt):
    build(opt)

    questions_path = os.path.join(
        opt['datapath'], 'FVQA', 'new_dataset_release', 'all_qs_dict_release.json'
    )
    trainset_path = os.path.join(opt['datapath'], 'FVQA', 'Name_Lists')
    image_path = os.path.join(
        opt['datapath'], 'FVQA', 'new_dataset_release', 'images', ''
    )

    return questions_path, trainset_path, image_path


class SplitTeacher(Teacher):
    """
    FVQA Teacher, which loads the json VQA data and implements its own `act` method for
    interacting with student agent.

    Use "fvqa:split:X" to choose between splits 0-4 (inclusive), or just "fvqa" to use
    the default split (0).
    """

    def __init__(self, opt, shared=None):
        super().__init__(opt)

        dt = opt['datatype'].split(':')[0]
        if dt not in ('train', 'test'):
            raise RuntimeError('Not valid datatype (only train/test).')

        task = opt.get('task', 'fvqa:split:0')
        task_num = 0  # default to train/split 0
        split = task.split(':')
        if len(split) > 2:
            task_num = split[2]
            if task_num not in [str(i) for i in range(5)]:
                raise RuntimeError('Invalid train/test split ID (0-4 inclusive)')

        if not hasattr(self, 'factmetrics'):
            if shared and shared.get('factmetrics'):
                self.factmetrics = shared['factmetrics']
            else:
                self.factmetrics = TeacherMetrics(opt.get('metrics', 'default'))
            self.datatype = opt['datatype']
        questions_path, trainset_path, self.image_path = _path(opt)

        if shared and 'ques' in shared:
            self.ques = shared['ques']
        else:
            self._setup_data(questions_path, trainset_path, dt, task_num)
        self.len = len(self.ques)

        self.asked_question = False
        # for ordered data in batch mode (especially, for validation and
        # testing), each teacher in the batch gets a start index and a step
        # size so they all process disparate sets of the data
        self.step_size = opt.get('batchsize', 1)
        self.data_offset = opt.get('batchindex', 0)
        self.image_loader = ImageLoader(opt)

        self.reset()

    def num_examples(self):
        return self.len

    def num_episodes(self):
        return self.len

    def report(self):
        r = super().report()
        for k, v in self.factmetrics.report().items():
            r[f'factmetrics_{k}'] = v
        return r

    def reset(self):
        # Reset the dialog so that it is at the start of the epoch,
        # and all metrics are reset.
        super().reset()
        self.lastY = None
        self.episode_idx = self.data_offset - self.step_size
        self.epochDone = False

    def reset_metrics(self):
        super().reset_metrics()
        self.factmetrics.clear()

    def observe(self, observation):
        """
        Process observation for metrics.
        """
        if self.lastY is not None:
            if self.asked_question:
                self.metrics.evaluate_response(observation, self.lastY[0])
            else:
                self.factmetrics.evaluate_response(observation, self.lastY[1])
                self.lastY = None
        return observation

    def act(self):
        if self.asked_question:
            self.asked_question = False
            action = {'text': 'Which fact supports this answer?', 'episode_done': True}
            if self.datatype.startswith('train'):
                action['labels'] = self.lastY[1]
            if (
                self.datatype != 'train'
                and self.episode_idx + self.step_size >= self.num_episodes()
            ):
                self.epochDone = True
            return action

        if self.datatype == 'train':
            self.episode_idx = random.randrange(self.len)
        else:
            self.episode_idx = (self.episode_idx + self.step_size) % self.num_episodes()

        self.asked_question = True
        qa = self.ques[self.episode_idx]
        question = qa['question']
        img_path = self.image_path + qa['img_file']

        action = {
            'image': self.image_loader.load(img_path),
            'text': question,
            'episode_done': False,
        }

        human_readable = qa['fact_surface'].replace('[', '').replace(']', '')
        self.lastY = [[qa['answer']], [human_readable]]

        if self.datatype.startswith('train'):
            action['labels'] = self.lastY[0]

        return action

    def share(self):
        shared = super().share()
        shared['factmetrics'] = self.factmetrics
        shared['ques'] = self.ques
        if hasattr(self, 'facts'):
            shared['facts'] = self.facts
        return shared

    def _setup_data(self, questions_path, trainset_path, datatype, task_num):
        print('loading: ' + questions_path)
        with PathManager.open(questions_path) as questions_file:
            questions = json.load(questions_file)
        train_test_images = set()
        fn = os.path.join(trainset_path, '{}_list_{}.txt'.format(datatype, task_num))
        with PathManager.open(fn) as imageset:
            for line in imageset:
                train_test_images.add(line.strip())
        self.ques = [
            questions[k]
            for k in sorted(questions.keys())
            if questions[k]['img_file'] in train_test_images
        ]


class DefaultTeacher(SplitTeacher):
    pass
